#include "stdafx.h"
#include "TccCard.h"
#include "hostspec.h"
#include "sys_gen.h"
#include "sys_pub.h"
#include "sys_priv.h"
#include "strips.h"

#include "message.h"
#include "dspcard.h"
#include "aldas.h"


IMPLEMENT_SERIAL( TccCard, CObject, 1 )



TccCard::TccCard(DSP* parent) {
	dsp=parent;
	clock=5;
	delay=128;
	impulse=1024;
	tfEmpty=FALSE;
	tfOver=FALSE;

	TRACE("TccCard created\n");
}

TccCard::~TccCard() {
	TRACE("TccCard destroyed\n");
}


CString & TccCard::GetName() {
	return name;
}

void TccCard::SetName(CString newname) {
	name = newname;
}


void TccCard::Serialize( CArchive& archive )
{
	char line[80];
	float version=1.1;

	if( archive.IsStoring() ) {
      sprintf (line, "# ALDAS %5.1f\n", VERSION);
      archive.WriteString(line);
		sprintf (line, "$%s\n", name);
		archive.WriteString (line);
		sprintf (line, "%i Alignment delay\n", delay);
		archive.WriteString (line);
		sprintf (line, "%i Impulse response\n", impulse);
		archive.WriteString (line);
		sprintf (line, "%i Clock\n", clock);
		archive.WriteString (line);
	}	else {
		archive.ReadString (line, 80);
		if (line[0]=='#') { // version 
			sscanf (line, "# ALDAS %f", &version);
			archive.ReadString(line, 80);
		}
		name=&(line[1]); // skip the $
		archive.ReadString (line, 80);
		sscanf (line, "%i", &delay);
		archive.ReadString (line, 80);
		sscanf (line, "%i", &impulse);
		archive.ReadString (line, 80);
		sscanf (line, "%i", &clock);
	}
}



int TccCard::GetTccCardStatus() {
	int status;
	dsp->ShortMessage(GET_TCC_STATUS, 0, 0, 0, 0);
	status = dsp->GetReply();
	delay = dsp->message_base[11];
	tfEmpty = dsp->message_base[12]!=0;
	tfOver = dsp->message_base[13]!=0;
	return status;
}


int TccCard::GetDelay()
{
	return delay;
}

void TccCard::SetDelay(int d)
{
	dsp->ShortMessage(SELECT_ALIGN_DELAY, 0, d, 0, 0);
	if (dsp->GetFlag(VERBOSE_FLAG) == 1) dsp->GetReply();
	delay=d;
}

int TccCard::GetClock()
{
	return clock;
}

void TccCard::SetClock(int c)
{
	dsp->ShortMessage(CHANGE_CLOCK, 0, c, 0, 0);
	if (dsp->GetFlag(VERBOSE_FLAG) == 1) dsp->GetReply();
	clock=c;
}

BOOL TccCard::GetTfEmpty()
{
	return tfEmpty;
}

BOOL TccCard::GetTfOver()
{
	return tfOver;
}

int TccCard::GetImpulse()
{
	return impulse;
}

void TccCard::SetImpulse(int i)
{
	dsp->ShortMessage(SELECT_TRIGGER_MODE, 0, i, 0, 0);
	if (dsp->GetFlag(VERBOSE_FLAG) == 1) dsp->GetReply();
	impulse = i;

}
