// ConfigDlg.cpp : implementation file
// This class is completely responsible for changes made
// to the hardware configuration while the program is
// running. When the user wants to save the current config,
// HAL will serialize the current configuration.

// Problems: There are four hardware classes, DSP, LLCard, TCC,
// and Module. They are connected by pointers. The problem is
// that when one object is destroyed, the pointers in the other
// objects must be nulled, or the other objects must be destroyed.

// And what do we do if a DSP is destroyed, and there are modules
// already attached? Do we leave the modules alive, or destroy them
// too?

// My solution: The hierarchy is this: DSP -> LLCard -> Module.
//														 -> TCC
// When an object is destroyed, all objects lower in the hierarchy must
// also be destroyed at the same time. A Module cannot exist without
// its' DSP and LLCard parents.

#include "stdafx.h"
#include <afxtempl.h>

#include "hal.h"

#include "dsp.h"

#include "message.h"
#include "dspcard.h"
#include "llcard.h"
#include "TccCard.h"
#include "module.h"
#include "lbic.h"

#include "aldas.h"
#include "ConfigDlg.h"


#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

// CConfigDlg dialog

CConfigDlg::CConfigDlg(CWnd* pParent /*=NULL*/, HAL * h)
	: CDialog(CConfigDlg::IDD, pParent)
{
	hal = h;
	//{{AFX_DATA_INIT(CConfigDlg)
	tccAttached = FALSE;
	dspLink0 = FALSE;
	dspLink1 = FALSE;
	dspLink2 = FALSE;
	dspLink3 = FALSE;
	//}}AFX_DATA_INIT
}

void CConfigDlg::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CConfigDlg)
	DDX_Control(pDX, IDC_ADD_DSP, addDspButton);
	DDX_Control(pDX, IDC_REMOVE_DSP, removeDspButton);
	DDX_Control(pDX, IDC_DSP_COMBO, dspListBox);
	DDX_Control(pDX, IDC_LLCARD_COMBO, llcardListBox);
	DDX_Control(pDX, IDC_BC_COMBO, biascardListBox);
	DDX_Control(pDX, IDC_MODULE_COMBO, moduleListBox);
	DDX_Check(pDX, IDC_TCC, tccAttached);
	DDX_Control(pDX, IDC_BUTTON3, dip1);
	DDX_Control(pDX, IDC_BUTTON4, dip2);
	DDX_Control(pDX, IDC_BUTTON5, dip3);
	DDX_Control(pDX, IDC_BUTTON6, dip4);
	DDX_Control(pDX, IDC_BUTTON7, dip5);
	DDX_Check(pDX, IDC_DSP_LINK0, dspLink0);
	DDX_Check(pDX, IDC_DSP_LINK1, dspLink1);
	DDX_Check(pDX, IDC_DSP_LINK2, dspLink2);
	DDX_Check(pDX, IDC_DSP_LINK3, dspLink3);
	//}}AFX_DATA_MAP
}

BOOL CConfigDlg::OnInitDialog() {
	CDialog::OnInitDialog();

	int numDsps = hal->dspArray.GetSize();
	DSP * dsp;
	for (int i=0; i<numDsps; i++) {
		dsp = hal->dspArray[i];
		dspListBox.AddString(dsp->GetName());
	}
	dspListBox.SetCurSel(0);

	if (numDsps==0) {
		address = 0;
	} else {
		address = hal->dspArray[0]->GetBaseAddress()/0x80000;
	}

	if (dspListBox.GetCurSel() != -1) {
		LLCard * llcard = hal->dspArray[0]->GetLLCard();
		if (!llcard) {
			llcardListBox.SetCurSel(0);
			moduleListBox.SetCurSel(0);
		}
		else {
			llcardListBox.SetCurSel(2);
			if (llcard->GetModule(0) || llcard->GetModule(1) || llcard->GetModule(2) || llcard->GetModule(3))
				moduleListBox.SetCurSel(1);
			else
				moduleListBox.SetCurSel(0);
		}
	}

	UpdateWindow();
	return TRUE;
}

BEGIN_MESSAGE_MAP(CConfigDlg, CDialog)
	//{{AFX_MSG_MAP(CConfigDlg)
	ON_BN_CLICKED(IDC_REMOVE_DSP, OnRemoveDsp)
	ON_CBN_SELCHANGE(IDC_DSP_COMBO, OnSelchangeDspCombo)
	ON_COMMAND_RANGE(IDC_BUTTON3, IDC_BUTTON7, OnDipButton)
	ON_BN_CLICKED(IDC_ADD_DSP, OnAddDsp)
	ON_CBN_SELCHANGE(IDC_LLCARD_COMBO, OnSelchangeLlcardCombo)
	ON_CBN_SELCHANGE(IDC_MODULE_COMBO, OnSelchangeModuleCombo)
	ON_BN_CLICKED(IDC_DSP_LINK0, OnDspLink0)
	ON_BN_CLICKED(IDC_DSP_LINK1, OnDspLink1)
	ON_BN_CLICKED(IDC_DSP_LINK2, OnDspLink2)
	ON_BN_CLICKED(IDC_DSP_LINK3, OnDspLink3)
	ON_BN_CLICKED(IDC_TCC, OnTcc)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

// CConfigDlg message handlers

void CConfigDlg::OnAddDsp() 
{
	DSP* myCard;
	int dipSetting = 0;
	int curAddress = 0;

	// Create a new DSP and add it to the dspArray.
	int numDsps = hal->dspArray.GetSize();
	myCard = new DSP(numDsps, dipSetting*0x80000, NULL);
	hal->dspArray.Add(myCard);
	
	// Select the first unused base address.
	// FIXME: this doesn't work right.
	while (curAddress == dipSetting*0x80000 && dipSetting<numDsps) {
		curAddress = hal->dspArray[dipSetting]->GetBaseAddress();
		dipSetting++;
	}
	hal->dspArray[dipSetting]->SetBaseAddress(dipSetting*0x80000);

	// Select an original name for the new DSP object.
	CString dspName("NewDsp");
	char buf[50];
	int name = 0;
	for (int i=0; i<numDsps; i++) {
		if (hal->dspArray[i]->GetName() == dspName)
			name++;
		sprintf(buf, "%s%i", "NewDsp", name);
		dspName = buf;
	}
//	sprintf(buf, "%s%i", dspName, name);
//	dspName = buf;
	int listLocation = dspListBox.AddString(dspName);
	dspListBox.SetCurSel(listLocation);
	myCard->SetName (dspName);


//	myCard->InitializeCard(LPCTSTR(hal->systemPath));

	UpdateWindow();
}

void CConfigDlg::OnRemoveDsp() 
{
	int i, j;
	int selection = dspListBox.GetCurSel();
	DSP * dsp = hal->dspArray[selection];

	// First, delete all the DSP's siblings, if any.
	LLCard * ll = dsp->GetLLCard();
	if (ll) {
		Module * m;
		for (i=0; i<4; i++) {
			m = ll->GetModule(i);
			if (m) {
				for (j=0; j<hal->moduleArray.GetSize(); j++)
					if (m == hal->moduleArray[j]) break;
				hal->moduleArray.RemoveAt(j);
			}
		}
	}

	// Now, delete the object, remove it from the hal array, and remove
	// it from the dialog item.
	delete dsp;
	hal->dspArray.RemoveAt(selection);
	dspListBox.DeleteString(selection);

	// Now set the current selection to the previous DSP in the list.
	int count = dspListBox.GetCount();
	if (selection < count)
		dspListBox.SetCurSel(selection);
	else dspListBox.SetCurSel(count);

	UpdateWindow();
}

// Add or remove a tcc card from the dsp here.
void CConfigDlg::OnTcc() 
{
	UpdateData();
	int old = tccAttached;

	int selection = dspListBox.GetCurSel();
	if (selection == -1) {
		tccAttached = FALSE;
		UpdateData(FALSE);
		UpdateWindow();
		return;
	}

	DSP * dsp = hal->dspArray[selection];

	TccCard * tcc = dsp->GetTccCard();
	if (tcc && tccAttached == FALSE) {
		delete tcc;
		dsp->RemoveTccCard();
	} else if (!tcc && tccAttached == TRUE) {
		TccCard * newCard = new TccCard(dsp);
		dsp->AttachTccCard(newCard);
	}

	UpdateWindow();
}

void CConfigDlg::UpdateWindow() {
	DSP * dsp = NULL;
	LLCard * llcard = NULL;
	Module * module = NULL;

	int sel = dspListBox.GetCurSel();
	if (sel >= 0) {
		GetDlgItem(IDC_TCC)->EnableWindow(TRUE);

		dsp = hal->dspArray[sel];
		address = dsp->GetBaseAddress()/0x80000;

		// Update the TCC check box.
		TccCard * tcc = dsp->GetTccCard();
		if (tcc) tccAttached = TRUE;
		else tccAttached = FALSE;
	} else {
		GetDlgItem(IDC_TCC)->EnableWindow(FALSE);
	}

	// If there are no more DSPs then, disable the remove dsp button.
	if (hal->dspArray.GetSize() == 0)
		removeDspButton.EnableWindow(FALSE);
	else removeDspButton.EnableWindow(TRUE);
	
	// There is a maximum of 16 DSPs in a VME crate, so set the upper
	// limit to 16.
	if (hal->dspArray.GetSize()>= 16)
		addDspButton.EnableWindow(FALSE);
	else addDspButton.EnableWindow(TRUE);

	// draw the dip switches
	CBitmapButton * dipButton;
	for (int j=0; j<5; j++) {
		dipButton = (CBitmapButton *) GetDlgItem(IDC_BUTTON3+j);
		if ((address >> (4-j)) & 1) {
			dipButton->LoadBitmaps(ID_DIP_SWITCH_UP, ID_DIP_SWITCH_MID);
		} else {
			dipButton->LoadBitmaps(ID_DIP_SWITCH_DOWN, ID_DIP_SWITCH_MID);
		}
		dipButton->Invalidate();
	}

	// Update the combo box for the llcard
	if (dsp) {
		GetDlgItem(IDC_LLCARD_COMBO)->EnableWindow(TRUE);
		llcard = dsp->GetLLCard();
	} else {
		GetDlgItem(IDC_LLCARD_COMBO)->EnableWindow(FALSE);
	}

	if (llcard) {
		GetDlgItem(IDC_MODULE_COMBO)->EnableWindow(TRUE);
		BOOL moduleAttached = FALSE;
		for (int i=0; i<4; i++) {
			module = llcard->GetModule(i);
			if (module) {
				moduleAttached = TRUE;
				switch (i) {
				case 0: dspLink0 = 1; break;
				case 1: dspLink1 = 1; break;
				case 2: dspLink2 = 1; break;
				case 3: dspLink3 = 1; break;
				}
			} else {
				switch (i) {
				case 0: dspLink0 = 0; break;
				case 1: dspLink1 = 0; break;
				case 2: dspLink2 = 0; break;
				case 3: dspLink3 = 0; break;
				}
			}
		}
	} else {
		GetDlgItem(IDC_MODULE_COMBO)->EnableWindow(FALSE);
		GetDlgItem(IDC_DSP_LINK0)->EnableWindow(FALSE);
		GetDlgItem(IDC_DSP_LINK1)->EnableWindow(FALSE);
		GetDlgItem(IDC_DSP_LINK2)->EnableWindow(FALSE);
		GetDlgItem(IDC_DSP_LINK3)->EnableWindow(FALSE);
		llcardListBox.SetCurSel(0);
	}
	if (moduleListBox.GetCurSel() == 1 && llcard) {
		GetDlgItem(IDC_DSP_LINK0)->EnableWindow(TRUE);
		GetDlgItem(IDC_DSP_LINK1)->EnableWindow(TRUE);
		GetDlgItem(IDC_DSP_LINK2)->EnableWindow(TRUE);
		GetDlgItem(IDC_DSP_LINK3)->EnableWindow(TRUE);
	}
	//llcardListBox.SetCurSel(2);	// Select the "Strips LLCard" text.

	// I also need to update the Bias Card and Module list boxes,
	// but right now, those classes aren't integrated into the
	// DSP, LLCard classes. So, we just disable that button for now.
	biascardListBox.SetCurSel(1);
	biascardListBox.EnableWindow(FALSE);

	UpdateData(FALSE);
}

void CConfigDlg::OnSelchangeDspCombo()
{
	UpdateWindow();
}

void CConfigDlg::OnSelchangeLlcardCombo() 
{
	int llType = llcardListBox.GetCurSel();
	CString llString;
	llcardListBox.GetLBText(llType, llString);

	// If the user selects 'strips' then add an llcard to the llcardArray,
	// and connect the DSP to the new llcard.
	if (llString == "Strips") {
		int dspSelect = dspListBox.GetCurSel();
		DSP * dsp;
		if (dspSelect >= 0) {
			dsp = hal->dspArray[dspSelect];
		} else return;
		
		LLCard * newcard = new LLCard(dsp);
		dsp->AttachLLCard(newcard);
	}

	// But if the user selects 'none' then remove the llcard from the llcardArray,
	// and disconnect it from the DSP, then delete it.
	else if (llString == "-none-") {
		int dspSelect = dspListBox.GetCurSel();
		DSP * dsp;
		if (dspSelect >= 0) {
			dsp = hal->dspArray[dspSelect];
		} else return;

		LLCard * card = dsp->GetLLCard();

		for (int i=0; i<4; i++) {
			Module * m = card->GetModule(i);
			for (int j=0; j<hal->moduleArray.GetSize(); j++)
				if (m == hal->moduleArray[j] && m) {
					hal->moduleArray.RemoveAt(j);
					break;
				}
		}

		dsp->RemoveLLCard();
		if (card) delete card;
	}
	UpdateWindow();
}

void CConfigDlg::OnSelchangeModuleCombo() 
{
	UpdateWindow();
}

afx_msg void CConfigDlg::OnDipButton(UINT nId) {
	int dipNumber = nId-IDC_BUTTON3;
	if (dipNumber < 0 || dipNumber > 5) return;
	dipNumber = 4 - dipNumber;

	int selection = dspListBox.GetCurSel();
	DSP * dsp = hal->dspArray[selection];
	address = dsp->GetBaseAddress()/0x80000;

	// Now change the address to the one selected by the user.
	address = address ^ (1 << dipNumber);
	dsp->SetBaseAddress(address*0x80000);

	UpdateWindow();
//	Invalidate();
}

void CConfigDlg::OnOK() 
{
//	CDialog::OnOK();
	// Check for valid dip switch settings here.
	// Also update the rest of the config. file.
	int numDsps = hal->dspArray.GetSize();
	int * addressArray = new int [numDsps];
	int duplication = -1;
	int i=0, j=0;
	DSP * dsp;
	for (i=0; i<numDsps; i++) {
		dsp = hal->dspArray[i];
		addressArray[i] = dsp->GetBaseAddress()/0x80000;
	}
	for (i=0; i<numDsps; i++) {
		for (j=0; j<numDsps; j++) {
			if (i==j) continue;
			if (addressArray[i]==addressArray[j]) {
				duplication = j;
				break;
			}
		}
		if (duplication>0) break;
	}

	delete addressArray;

	char buf[500];

	if (duplication != -1) {
		dsp = hal->dspArray[duplication];	
		// perhaps insufficient. this only catches one redundant address
		i = sprintf(buf, "DSP addresses must be unique. Please\n");
		i += sprintf(buf, "change %s to a new address\n", dsp->GetName());
		AfxMessageBox(buf, MB_ICONSTOP);
	} else {
		CDialog::OnOK();
	}

	// And finally initialize the system before closing the box.
	hal->InitializeSystem();
}

/*			if (moduleAttached)
				moduleListBox.SetCurSel(1);
			else
				moduleListBox.SetCurSel(0);
*/

// If there is a module attached to the selected link of the current llcard
// then we delete it. If there isn't, then we add one.
void CConfigDlg::AddRemoveModule(int link) {
	int dspSelect = dspListBox.GetCurSel();
	DSP * dsp;
	if (dspSelect >= 0) dsp = hal->dspArray[dspSelect];
	else return;

	int llSelect = llcardListBox.GetCurSel();
	LLCard * llcard;
	CString llString;
	llcardListBox.GetLBText(llSelect, llString);

	if (llString == "Strips") llcard = dsp->GetLLCard();
	else return;

	int moduleSelect = moduleListBox.GetCurSel();
	CString moduleString;
	moduleListBox.GetLBText(moduleSelect, moduleString);

	Module * module;
	if (moduleString == "LBIC+CDP") {
		// Determine whether a module is attached to the llcard on this link
		// If not, then add a new one.
		if (llcard && !llcard->GetModule(link)) {
			module = new LBIC;
			module->link = link;
			module->llCard = llcard;

			hal->moduleArray.InsertAt(0, module);
			llcard->AttachModule(module, link);
		}
		// If a module is attached, remove it from the llcard, the moduleArray,
		// and delete the module itself.
		else if (llcard && llcard->GetModule(link)) {
			module = llcard->GetModule(link);

			for (int i=0; i<hal->moduleArray.GetSize(); i++)
				if (module == hal->moduleArray[i]) break;
			hal->moduleArray.RemoveAt(i);
			delete module;
			llcard->RemoveModule(link);
		}
		// If there is no llcard, then simply add the module to moduleArray
		else if (!llcard) {
			module = new LBIC;
			module->link = link;

			hal->moduleArray.InsertAt(0, module);
		}
	}

	UpdateData(FALSE);
}

void CConfigDlg::OnDspLink0() 
{
	int old = dspLink0;
	UpdateData();
	if (dspLink0 == old)
		dspLink0 = -1;
	UpdateData(FALSE);

	AddRemoveModule(0);
}

void CConfigDlg::OnDspLink1() 
{
	int old = dspLink1;
	UpdateData();
	if (dspLink1 == old)
		dspLink1 = -1;
	UpdateData(FALSE);

	AddRemoveModule(1);
}

void CConfigDlg::OnDspLink2() 
{
	int old = dspLink2;
	UpdateData();
	if (dspLink2 == old)
		dspLink2 = -1;
	UpdateData(FALSE);

	AddRemoveModule(2);
}

void CConfigDlg::OnDspLink3() 
{
	int old = dspLink3;
	UpdateData();
	if (dspLink3 == old)
		dspLink3 = -1;
	UpdateData(FALSE);

	AddRemoveModule(3);
}
