// CollectDataDoc.cpp : implementation of the CCollectDataDoc class

#include "stdafx.h"
#include <afxtempl.h>
#include "ALDAS.h"

#include "PlaceDoc.h"
#include "CollectDataDoc.h"
#include "SelectDspDlg.h"
#include "ControlBlockDlg.h"
#include "TriggerSourceDlg.h"
#include "TriggersDlg.h"
#include "TakeDataDlg.h"
#include "Data.h"
#include "Message.h"
#include "DSPcard.h"
#include "llcard.h"
#include "DspDlg.h"
#include "module.h"

#include "hal.h"

#include "SelectModuleDlg.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

extern CALDASApp theApp;

// CCollectDataDoc

IMPLEMENT_DYNCREATE(CCollectDataDoc, CDocument)

BEGIN_MESSAGE_MAP(CCollectDataDoc, CDocument)
	//{{AFX_MSG_MAP(CCollectDataDoc)
		// NOTE - the ClassWizard will add and remove mapping macros here.
		//    DO NOT EDIT what you see in these blocks of generated code!
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

// CCollectDataDoc construction/destruction

CCollectDataDoc::CCollectDataDoc()
{
	// TODO: add one-time construction code here
	link=0;
	data=NULL;
	badChannel = NULL;
	heading.Empty();
	place.length=sizeof (WINDOWPLACEMENT);
	place.showCmd=SW_NORMAL; //nCmdShow;
	place.flags=WPF_SETMINPOSITION;
	place.ptMinPosition=CPoint(0,0);
	CRect client;
	AfxGetMainWnd()->GetClientRect (client);
	place.ptMaxPosition=CPoint(client.left, client.bottom); 
	place.rcNormalPosition=CRect(-1, -1, client.Width()/4*3, client.Height()/4*3);

	hal = theApp.GetHal();

	TRACE("CCollectDataDoc created\n");
}

CCollectDataDoc::~CCollectDataDoc()
{
	if (data) delete data;
	delete [] badChannel;
	TRACE("CCollectDataDoc destroyed\n");
}

BOOL CCollectDataDoc::OnNewDocument()
{
//	int controlBlock;
	int i; 

	if (!CDocument::OnNewDocument())
		return FALSE;

	CSelectModuleDlg dlg;
	if (dlg.DoModal() != IDOK) return FALSE;
	int moduleIndex = dlg.moduleNumber;

	module = hal->GetModule(moduleIndex);
	moduleName = module->GetModuleName();

	hal->ModifySettings(moduleIndex);

	CTriggerSourceDlg tdlg;
	if (tdlg.DoModal() != IDOK) return FALSE;

	numChannels=module->GetNumChannels();
	badChannel=new BOOL [numChannels];
	for (i=0; i<numChannels; i++) badChannel[i]=module->badChannel[i];

	data = new CData(moduleIndex);

	CTakeDataDlg take;
	take.Init(data, moduleIndex, tdlg.triggers, tdlg.triggerSource);
	if (take.DoModal()!=IDOK)  return FALSE; // canceled

	heading.Format("Collect Data: %d Triggers, DSP %s, Link %d", 
						data->GetTriggers(), data->dsp->GetName(), data->GetLink());
	SetModifiedFlag(TRUE);
	return TRUE;
}

// CCollectDataDoc serialization

void CCollectDataDoc::Serialize(CArchive& ar)
{
  CString buf;
  char line[100];
  float version=1.1;
  int i, j, numBad;

	if (ar.IsStoring())	{    // storing code here:
	  buf.Format ("# ALDAS %5.1f\n", VERSION);
	  ar.WriteString(buf);
     buf.Format ("$%s\n", heading);
     ar.WriteString(buf);
     buf.Format ("%i\n", link);
     ar.WriteString (buf);
     data->Serialize(ar);
	  				// count # of bad channels:
		numBad=0;
		for (i=0; i< numChannels; i++) 
			if (badChannel[i]) numBad++;
		sprintf (line, "%i bad channels\n", numBad);
		ar.WriteString (line);

		for (i=0; i< numChannels; i++) {
			if (badChannel[i]) {
				sprintf (line, "%i is bad\n", i);
				ar.WriteString (line);
			}
		}

	}	else	{		//loading code here
		ar.ReadString(line, 100);
		if (line[0]=='#') { // version 
			sscanf (line, "# ALDAS %f", &version);
			ar.ReadString(line, 100);
		}
		heading=&(line[1]); // skip '$'
		ar.ReadString (line, 100);
		sscanf(line, "%i", &link);
		data = new CData(NULL, link);
		data->Serialize(ar);
		badChannel=new BOOL [data->GetNumChannels()];
		if (version>=1.2) {
			ar.ReadString (line, 80);
			sscanf (line, "%i", &numBad);
			for (i=0; i< data->GetNumChannels(); i++) badChannel[i]=FALSE;
			for (i=0; i< numBad; i++) {
				ar.ReadString (line, 80);
				sscanf (line, "%i", &j);
				badChannel[j]=TRUE;
			}
		}
	}
}

CString CCollectDataDoc::remove_path(const char * mystring) {
  const char * cur = mystring;
  char t[40];
  int i = 0;

  while (*cur != '\0') {
    if (*cur == '\\' || *cur == ':')
      i = 0;
    else             // Else copy this character and increment the ptrs.
      t[i++] = *cur;
    cur++;
  }
  t[i] = '\0';

  CString cs(t);
  return cs;
}


// CCollectDataDoc diagnostics

#ifdef _DEBUG
void CCollectDataDoc::AssertValid() const
{
	CDocument::AssertValid();
}

void CCollectDataDoc::Dump(CDumpContext& dc) const
{
	CDocument::Dump(dc);
}
#endif //_DEBUG

// CCollectDataDoc commands

