// DelayScanDoc.cpp : implementation of the CCollectDataDoc class
//

#include "stdafx.h"
#include <afxtempl.h>
#include <math.h>
#include "ALDAS.h"
#include "MainFrm.h"

#include "PlaceDoc.h"
#include "ScanDoc.h"
#include "Axis.h"
#include "DelayScanDoc.h"
#include "DelayScanDlg.h"
#include "TakeDataDlg.h"
#include "Data.h"
#include "Message.h"
#include "DSPcard.h"
#include "llcard.h"
#include "module.h"

#include "hal.h"

#include "SelectModuleDlg.h"
//#include "TriggerSourceDlg.h"

//#include "fitLSQ.h"


#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

extern CALDASApp theApp;

// CCollectDataDoc

IMPLEMENT_DYNCREATE(CDelayScanDoc, CDocument)

BEGIN_MESSAGE_MAP(CDelayScanDoc, CDocument)
	//{{AFX_MSG_MAP(CDelayScanDoc)
	ON_COMMAND(ID_FILE_CLOSE, OnFileClose)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

// CCollectDataDoc construction/destruction

CDelayScanDoc::CDelayScanDoc()
{
	numParam=2;
	axis.SetSize(numParam);
	par.SetSize(numParam);
	err.SetSize(numParam);
	chi2.SetSize(1);
}

CDelayScanDoc::~CDelayScanDoc()
{
}

BOOL CDelayScanDoc::OnNewDocument()
{
	int i, j, k;
	UINT triggers;
	CString cbOld; // name of old control block # 3
	CMainFrame* pFrame = (CMainFrame*) AfxGetApp()->m_pMainWnd;
	CStatusBar* pStatus= &pFrame->m_wndStatusBar;
	CString buf;

	if (!CDocument::OnNewDocument())
		return FALSE;
	

	CSelectModuleDlg dlg;
	if (dlg.DoModal() != IDOK) return FALSE;
	int moduleIndex = dlg.moduleNumber;

	module = hal->GetModule(moduleIndex);
	moduleName = module->GetModuleName();

	hal->ModifySettings(moduleIndex);



	///////////////// Get min, max, and increment. //////////////////
	CDelayScanDlg delayDlg;
/*	delayDlg.increment=1;
	delayDlg.minimum=210;
	delayDlg.maximum=240;
	delayDlg.threshold=0.1;
	delayDlg.triggers=100;*/
	if (delayDlg.DoModal() != IDOK)	return FALSE;

	increment = delayDlg.increment;
	max = delayDlg.maximum;
	min = delayDlg.minimum;
	numPoints = (UINT) ((max - min)/increment + 1.5);
	dataArray.SetSize(numPoints);
	triggers = delayDlg.triggers;


	numChannels=hal->GetNumChannels(moduleIndex);
	if (numChannels<1) return FALSE;
	badChannel=new BOOL [numChannels];
	for (i=0; i<numChannels; i++) badChannel[i]=module->badChannel[i];

	CTakeDataDlg take;
//	take.maxTriggers = triggers;

	int oldDelay = hal->GetAlignmentDelay(moduleIndex);
	float oldThreshold = hal->GetThreshold(moduleIndex);
	int curDelay = min;
	hal->SetThreshold(moduleIndex, delayDlg.threshold);
	for (j=0; j<numPoints; j++) {
		hal->SetAlignmentDelay(moduleIndex, curDelay);
		dataArray[j] = new CData (moduleIndex);
//		for (k=0; k<numChannels; k++)	dataArray[j]->hits[k] = 0;
		buf.Format("Delay scan %i of %i: Delay = %i clock cycles.\n", j+1, numPoints, curDelay);
		pStatus->SetPaneText (0, buf);
		take.Init(dataArray[j], moduleIndex, triggers, HAL::EXTERNAL);
		if (take.DoModal()!=IDOK)  {
			hal->SetAlignmentDelay(moduleIndex, oldDelay);
			hal->SetThreshold(moduleIndex, oldThreshold); 
			pStatus->SetPaneText (0, "");
			return FALSE; // canceled
		}
		curDelay += increment;
	}

	pStatus->SetPaneText (0, "");
	hal->SetAlignmentDelay(moduleIndex, oldDelay);
	hal->SetThreshold(moduleIndex, oldThreshold); 
	AnalyzeData();

	heading.Format("Delay Scan of '%s', %d Triggers", 
						module->GetModuleName(), dataArray[0]->GetTriggers());

	SetModifiedFlag(TRUE);
	return TRUE;
}


void CDelayScanDoc::AnalyzeData() {
	int i, j, trigs, hits, doubleHits;
	float curDelay;
	int doubleCount = 0;

	par[0] = new float [numPoints]; // efficiency
	par[1] = new float [numPoints]; // delay
	err[1] = new float [numPoints]; // delay
	err[0] = new float [numPoints]; // efficiency
	chi2[0] = new float [numPoints];

	// Average all channels together to get the mean efficiency for
	// each alignment delay.
	// Also, calculate the ratio of single to double hits.
	curDelay = min;
	hitRatio = 0;
	for (j=0; j<numPoints; j++) {
		trigs = dataArray[j]->GetTriggers();
		hits = 0;
		doubleHits = 0;
		for (i=0; i<numChannels; i++) {
			if (!module->badChannel[i]) {
				hits += dataArray[j]->hits[i];
				doubleHits += dataArray[j]->multipleHits[i];
			}
		}
		if (hits > 0 && doubleHits > 0) {
			hitRatio += float(hits) / float(doubleHits) - 1;
			doubleCount++;
		}

		if (trigs == 0) {
			par[0][j] = 0;
			err[0][j] = 0;
		} else {
			par[0][j] = float (hits)/float(trigs*numChannels);
			err[0][j] = sqrt(par[0][j]*(1.0-par[0][j])/(trigs*numChannels));
		}

		par[1][j] = curDelay;
		err[1][j] = 0.0;
		chi2[0][j] = 0.0;
		curDelay += increment;
	}

	hitRatio = float(hitRatio) / float(doubleCount);

	axis[0]=new CAxis ("Efficiency", numPoints, par[0], err[0], NULL);
	axis[1]=new CAxis ("Delay", numPoints, par[1], err[1], NULL);
}


// CDelayScanDoc diagnostics

#ifdef _DEBUG
void CDelayScanDoc::AssertValid() const
{
	CDocument::AssertValid();
}

void CDelayScanDoc::Dump(CDumpContext& dc) const
{
	CDocument::Dump(dc);
}
#endif //_DEBUG

// CDelayScanDoc commands

void CDelayScanDoc::OnFileClose() 
{
	// TODO: Add your command handler code here
	
}

void CDelayScanDoc::Serialize(CArchive & ar)
{
  int i, j, numBad;
  CString buf;
  char line[100], name[100];
  float version=1.1;

	if (ar.IsStoring())	{
    // storing code here:
		buf.Format ("# ALDAS %5.1f\n", VERSION);
		ar.WriteString(buf);
		buf.Format ("$%s\n", heading);
		ar.WriteString(buf);
		if (dataArray[0] != NULL)
			buf.Format ("%i %i %i\n", link, dataArray[0]->GetTriggers(), 
				dataArray[0]->GetNumChannels());
		else {
			TRACE ("CDelayScanDoc::Serialize: dataArray[0] == NULL\n");
			return;			// FIXME: serialization has not completed 
		}                 //        so an error flag should be set.
		ar.WriteString (buf);	
		buf.Format ("%i numPoints\n", numPoints);
		ar.WriteString (buf);

		for (i=0; i<numPoints; i++) {
			dataArray[i]->Serialize(ar);
		}
		buf.Format ("%i numParam\n", numParam);
		ar.WriteString (buf);
		buf.Format ("%i numChi2\n", numChi2);
		ar.WriteString (buf);
		for (i=0; i<numChi2; i++){
			for (j=0; j<numPoints; j++) {
				buf.Format ("%f\n", chi2[i][j]);
				ar.WriteString(buf);
			}
		}
		for (i=0; i<numParam; i++) { // loop over parameters
			buf.Format ("$%s\n", axis[i]->GetLabel());
			ar.WriteString(buf);
			for (j=0; j<numPoints; j++) {
				buf.Format ("%f\t%f\n", par[i][j], err[i][j]);
				ar.WriteString(buf);
			}
		}
		if (VERSION>=1.2) {
			buf.Format ("$%s\n", moduleName);
			ar.WriteString(buf);
		}
		// count # of bad channels:
		numBad=0;
		for (i=0; i< numChannels; i++) 
			if (badChannel[i]) numBad++;
		sprintf (line, "%i bad channels\n", numBad);
		ar.WriteString (line);

		for (i=0; i< numChannels; i++) {
			if (badChannel[i]) {
				sprintf (line, "%i is bad\n", i);
				ar.WriteString (line);
			}
		}

	} 	else {   //loading code here		
		ar.ReadString(line, 100);
		if (line[0]=='#') { // version 
			sscanf (line, "# ALDAS %f", &version);
			ar.ReadString(line, 100);
		}
		heading=&(line[1]); // skip '$'

		ar.ReadString (line, 100);
		sscanf(line, "%i %i %i", &link, &trig, &numChannels);

		// delete old data points:
		for (i=0; i<numPoints; i++) {
			delete dataArray[i];
		}

		ar.ReadString (line, 100);
		sscanf(line, "%i", &numPoints);
		dataArray.SetSize(numPoints);
		for (i=0; i<numPoints; i++) {
			dataArray[i] = new CData(NULL, link);
			dataArray[i]->Serialize(ar);
		}
		ar.ReadString (line, 100);
		sscanf(line, "%i", &numParam);

		// delete old results:
		for (i=0; i<axis.GetSize(); i++) delete   axis[i];
		for (i=0; i<par.GetSize();  i++) delete [] par[i];
		for (i=0; i<err.GetSize();  i++) delete [] err[i];
		for (i=0; i<chi2.GetSize(); i++) delete [] chi2[i];
		
		if (version>=1.2) {
			ar.ReadString (line, 100);
			sscanf(line, "%i", &numChi2);
		} else {
			numChi2 = 1;
		}
		par.SetSize (numParam);
		err.SetSize (numParam);
		axis.SetSize (numParam);
		chi2.SetSize (numChi2);

		for (i=0; i<numParam; i++) {
			par[i] = new float [numPoints];
			err[i] = new float [numPoints];
		}
		for (i=0; i<numChi2; i++){
			chi2[i] = new float [numPoints];
			for (j=0; j<numPoints; j++) {
				ar.ReadString (line, 100);
				sscanf(line, "%f", &(chi2[i][j]));
			}
		}
		for (i=0; i<numParam; i++) {
			ar.ReadString (name, 100);
			for (j=0; j<numPoints; j++) {
				ar.ReadString (line, 100);
				sscanf(line, "%f %f", &(par[i][j]), &(err[i][j]));
			}
			axis[i]= new CAxis(&(name[1]), numPoints, par[i], err[i], chi2[0]);
		}
		if (version>=1.2) {
			ar.ReadString (line, 100);
			moduleName=&(line[1]); // skip '$'
		} else {
			moduleName.Empty();
		}
		badChannel=new BOOL [numChannels];
		if (version>=1.2) {
			ar.ReadString (line, 80);
			sscanf (line, "%i", &numBad);
			for (i=0; i< numChannels; i++) badChannel[i]=FALSE;
			for (i=0; i< numBad; i++) {
				ar.ReadString (line, 80);
				sscanf (line, "%i", &j);
				badChannel[j]=TRUE;
			}
		}
	}
}