// NoiseScanDoc.cpp : implementation of the CCollectDataDoc class
//

#include "stdafx.h"
#include <afxtempl.h>
#include <math.h>
#include "ALDAS.h"
#include "MainFrm.h"

#include "PlaceDoc.h"
#include "ScanDoc.h"
#include "Axis.h"
#include "NoiseScanDoc.h"
#include "NoiseScanDlg.h"
#include "TakeDataDlg.h"
#include "Data.h"
#include "Message.h"
#include "DSPcard.h"
#include "llcard.h"
#include "module.h"

#include "hal.h"

#include "SelectModuleDlg.h"
#include "TriggerSourceDlg.h"

#include "fitLSQ.h"


#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

extern CALDASApp theApp;

// CCollectDataDoc

IMPLEMENT_DYNCREATE(CNoiseScanDoc, CDocument)

BEGIN_MESSAGE_MAP(CNoiseScanDoc, CDocument)
	//{{AFX_MSG_MAP(CNoiseScanDoc)
	ON_COMMAND(ID_FILE_CLOSE, OnFileClose)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

// CCollectDataDoc construction/destruction

CNoiseScanDoc::CNoiseScanDoc()
{
	numParam=2;
	axis.SetSize(2);
	par.SetSize(2);
	err.SetSize(2);
	chi2.SetSize(1);
}

CNoiseScanDoc::~CNoiseScanDoc()
{
}

BOOL CNoiseScanDoc::OnNewDocument()
{
	int i, j, k;
	UINT triggers;
	CString cbOld; // name of old control block # 3
	CMainFrame* pFrame = (CMainFrame*) AfxGetApp()->m_pMainWnd;
	CStatusBar* pStatus= &pFrame->m_wndStatusBar;
	CString buf;

	if (!CDocument::OnNewDocument())
		return FALSE;
	

	CSelectModuleDlg dlg;
	if (dlg.DoModal() != IDOK) return FALSE;
	int moduleIndex = dlg.moduleNumber;

	module = hal->GetModule(moduleIndex);
	moduleName = module->GetModuleName();

	hal->ModifySettings(moduleIndex);

	CTriggerSourceDlg tdlg;
	if (tdlg.DoModal() != IDOK) return FALSE;
	triggers = tdlg.triggers;


	///////////////// Get min, max, and increment. //////////////////
	CNoiseScanDlg noiseDlg;
	if (noiseDlg.DoModal() != IDOK)	return FALSE;

	increment = noiseDlg.mIncrement;
	max = noiseDlg.mMaximumThreshold;
	min = noiseDlg.mMimimumThreshold;
	numPoints = (UINT) ((max - min)/increment + 1.5);
	dataArray.SetSize(numPoints);


	numChannels=hal->GetNumChannels(moduleIndex);

	if (numChannels<1) return FALSE;
	par[0] = new float [numChannels];
	par[1] = new float [numChannels];
	err[0] = new float [numChannels];
	err[1] = new float [numChannels];
	chi2[0] = new float [numChannels];
	badChannel=new BOOL [numChannels];
	for (i=0; i<numChannels; i++) badChannel[i]=module->badChannel[i];

	CTakeDataDlg take;
	take.maxTriggers = triggers;

	float oldThreshold;
	float curThreshold = min;
	float oldCalibration = hal->GetCalibration(moduleIndex);
	hal->SetCalibration(moduleIndex, 0);
	oldThreshold = hal->GetThreshold(moduleIndex);

	for (j=0; j<numPoints; j++) {
		hal->SetThreshold(moduleIndex, curThreshold);
		// FIXME: this is a test. this is only a test.
//		for (k=0; k<4; k++) hal->SetThreshold(k, curThreshold);

		dataArray[j] = new CData (moduleIndex);
//		for (k=0; k<numChannels; k++)	dataArray[j]->hits[k] = 0;
		buf.Format("Noise scan %i of %i: Threshold = %f Volt.\n", j+1, numPoints, curThreshold);
		pStatus->SetPaneText (0, buf);
		take.Init(dataArray[j], moduleIndex, triggers, tdlg.triggerSource);
		if (take.DoModal()!=IDOK)  {
			hal->SetThreshold(moduleIndex, oldThreshold);
			hal->SetCalibration(moduleIndex, oldCalibration);
			pStatus->SetPaneText (0, "");
			return FALSE; // canceled
		}
		curThreshold += increment;
	}

	pStatus->SetPaneText (0, "");
	hal->SetThreshold(moduleIndex, oldThreshold); 
	hal->SetCalibration(moduleIndex, oldCalibration);
	AnalyzeData();

	heading.Format("Noise Scan of '%s', %d Triggers", 
						module->GetModuleName(), dataArray[0]->GetTriggers());

	SetModifiedFlag(TRUE);
	return TRUE;
}


void CNoiseScanDoc::AnalyzeData() {
	int i, j;
	float curThreshold = min;
	float *tempY, *tempX, *weight;
	tempY = new float [numPoints];
	tempX = new float [numPoints];
	weight = new float [numPoints];
	UINT hits = 0;
	float ave_eff;
	int trigs = 0;
	float q, a, b, e1, e2, chi;
	int nonzero; // count # of points with >1 hits
	

	for (i=0; i<numChannels; i++) {
		par[0][i] = 0;
		par[1][i] = 0;
		if (module->badChannel[i]) {
			chi2[0][i] = -2;
		} else {
			nonzero = 0;
			curThreshold = min;
			for (j=0; j<numPoints; j++) {
				trigs = dataArray[j]->GetTriggers();
				hits = dataArray[j]->hits[i];
				ave_eff = float (hits)/float(trigs);
				if (hits != 0) {
					weight[j] = ave_eff/sqrt(ave_eff*(1.0-ave_eff)/trigs); // Binomial stat.
					nonzero++;
					tempY[j] = log(ave_eff);
				} else {
					weight[j] = 0;
					tempY[j] = 0;
				}
				tempX[j] = curThreshold * curThreshold;
				curThreshold += increment;
			}
			if (nonzero>1) { // more than 1 non-zero efficiency
				FitLine(tempX, tempY, numPoints, weight, 1, &a, &b, &e1, &e2, &chi, &q);
				par[0][i] = a;
				par[1][i] = b;
				err[0][i] = e1;
				err[1][i] = e2;
				chi2[0][i] = chi;
			} else {
				chi2[0][i] = -1;
			}
		}
	}

	delete [] tempY;
	delete [] tempX;
	delete [] weight;

	axis[0]=new CAxis ("Y Intercept", numChannels, par[0], err[0], chi2[0]);
	axis[1]=new CAxis ("Slope", numChannels, par[1], err[1], chi2[0]);
}


// CNoiseScanDoc diagnostics

#ifdef _DEBUG
void CNoiseScanDoc::AssertValid() const
{
	CDocument::AssertValid();
}

void CNoiseScanDoc::Dump(CDumpContext& dc) const
{
	CDocument::Dump(dc);
}
#endif //_DEBUG

// CNoiseScanDoc commands

void CNoiseScanDoc::OnFileClose() 
{
	// TODO: Add your command handler code here
	
}
