// NoiseScanView.cpp : implementation file
//
#include "stdafx.h"
#include <afxtempl.h>
#include "ALDAS.h"
#include "MainFrm.h"
#include <math.h>

#include "PlaceDoc.h"
#include "ScanDoc.h"
#include "Axis.h"
#include "ScanView.h"
#include "HeadingDlg.h"
#include "NoiseFitDlg.h"
#include "Data.h"
#include "FitFunction.h"
#include "NoiseScanView.h"
#include "NoiseScanDoc.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CNoiseScanView

IMPLEMENT_DYNCREATE(CNoiseScanView, CScrollView)

CNoiseScanView::CNoiseScanView()
{
}

CNoiseScanView::~CNoiseScanView()
{
}


BEGIN_MESSAGE_MAP(CNoiseScanView, CScrollView)
	//{{AFX_MSG_MAP(CNoiseScanView)
	ON_WM_MOUSEMOVE()
	ON_WM_LBUTTONDOWN()
	ON_COMMAND(ID_HISTOGRAM_VIEW, OnHistogramView)
	ON_UPDATE_COMMAND_UI(ID_HISTOGRAM_VIEW, OnUpdateHistogramView)
	ON_COMMAND(ID_LINE_FIT_VIEW, OnLineFitView)
	ON_UPDATE_COMMAND_UI(ID_LINE_FIT_VIEW, OnUpdateLineFitView)
	ON_COMMAND(ID_SETHEADING, OnSetheading)
	//}}AFX_MSG_MAP
	ON_COMMAND(ID_FILE_PRINT, CView::OnFilePrint)
	ON_COMMAND(ID_FILE_PRINT_DIRECT, CView::OnFilePrint)
	ON_COMMAND(ID_FILE_PRINT_PREVIEW, CView::OnFilePrintPreview)
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CNoiseScanView diagnostics

#ifdef _DEBUG
void CNoiseScanView::AssertValid() const
{
	CScrollView::AssertValid();
}

void CNoiseScanView::Dump(CDumpContext& dc) const
{
	CScrollView::Dump(dc);
}

CNoiseScanDoc* CNoiseScanView::GetDocument() // non-debug version is inline
{
//	ASSERT(m_pDocument->IsKindOf(RUNTIME_CLASS(CScanDoc)));
	return (CNoiseScanDoc*)m_pDocument;
}
#endif //_DEBUG

/////////////////////////////////////////////////////////////////////////////
// CNoiseScanView message handlers

void CNoiseScanView::OnMouseMove(UINT nFlags, CPoint point) 
{	
	CScanView::OnMouseMove(nFlags, point);
}

void CNoiseScanView::OnLButtonDown(UINT nFlags, CPoint point) 
{
	int ch; // channel number
	int i, link;
	float triggers; 
	CNoiseScanDoc* pDoc = GetDocument();
	CAxis *xAxis, *yAxis;

	if (viewMode == LineFitView) {
		CPoint scroll = GetScrollPosition( );
		point=point+scroll; // add scroll offset to convert from window to document
		ch=pDoc->axis[0]->GetChannelNumber(point);
		link = pDoc->dataArray[0]->GetLink();
		if (ch>=0 && (pDoc->axis[0]->CursorInsideGraphRect(point) ||
			pDoc->axis[1]->CursorInsideGraphRect(point))) {
			// get the rawdata to be displayed:
			xTemp=new float[pDoc->numPoints];
			yTemp=new float[pDoc->numPoints];
			eTemp=new float[pDoc->numPoints];
			for (i=0; i<pDoc->numPoints; i++){
				triggers=pDoc->dataArray[i]->GetTriggers();
				if (triggers<=0) triggers=1.0;
				yTemp[i]=pDoc->dataArray[i]->hits[ch]/triggers;
//				if (yTemp[i]>0) yTemp[i]=log10(yTemp[i]);
				eTemp[i]=sqrt(yTemp[i]*(1-yTemp[i])/triggers);
				xTemp[i]=pDoc->dataArray[i]->GetLLDacVoltage(link);
				xTemp[i]*=xTemp[i];
			}
			yAxis=new CAxis("Hits", pDoc->numPoints, yTemp, eTemp, &pDoc->axis[0]->GetChiSquare()[ch]);
			xAxis=new CAxis("Threshold squared", pDoc->numPoints, xTemp, NULL, &pDoc->axis[0]->GetChiSquare()[ch]);
			yAxis->SetType(LogAxis);

			CNoiseFitDlg dlg;
			CFitFunction* func = 	new CFitFunction( LineType, pDoc->axis[0]->GetData()[ch], pDoc->axis[1]->GetData()[ch]);
			dlg.Init(pDoc->axis[0], pDoc->axis[1], ch, pDoc->badChannel, xAxis, yAxis, func);
			if (dlg.DoModal()==IDOK) {
				pDoc->badChannel[i] = dlg.badChannelCheck;
			}
			delete[] xTemp;
			delete[] yTemp;
			delete[] eTemp;
			delete func;

			delete xAxis;
			delete yAxis;
			xAxis=NULL;
			yAxis=NULL;
			xTemp=NULL;
			yTemp=NULL;
			eTemp=NULL;
		}
	}
	CScrollView::OnLButtonDown(nFlags, point);
}

void CNoiseScanView::OnHistogramView() 
{
	CScanView::OnHistogramView();
}

void CNoiseScanView::OnUpdateHistogramView(CCmdUI* pCmdUI) 
{
	CScanView::OnUpdateHistogramView(pCmdUI);
}

void CNoiseScanView::OnLineFitView() 
{
	CScanView::OnLineFitView();
}

void CNoiseScanView::OnUpdateLineFitView(CCmdUI* pCmdUI) 
{
	CScanView::OnUpdateLineFitView(pCmdUI);
}

void CNoiseScanView::OnSetheading() 
{
	CScanView::OnSetheading();
}
