// ScanDoc.cpp : implementation file
//

#include "stdafx.h"
#include <afxtempl.h>
#include "ALDAS.h"

#include "PlaceDoc.h"
#include "ScanDoc.h"
#include "Axis.h"
#include "SelectDspDlg.h"
#include "DSPDlg.h"
#include "ThresholdDlg.h"
#include "TakeDataDlg.h"
#include "Data.h"
#include "Message.h"
#include "DSPcard.h"
#include "llcard.h"
#include "module.h"

#include "hal.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

extern CALDASApp theApp;

#define new DEBUG_NEW

// CScanDoc

IMPLEMENT_DYNCREATE(CScanDoc, CDocument)

BEGIN_MESSAGE_MAP(CScanDoc, CDocument)
	//{{AFX_MSG_MAP(CScanDoc)
		// NOTE - the ClassWizard will add and remove mapping macros here.
		//    DO NOT EDIT what you see in these blocks of generated code!
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

// CCollectDataDoc construction/destruction

CScanDoc::CScanDoc()
{
	// TODO: add one-time construction code here
	numPoints = 0;
	link=0;
	heading.Empty();
	moduleName.Empty();
	badChannel=NULL; 
	numParam=0;
	numChi2=1;
	place.length=sizeof (WINDOWPLACEMENT);
	place.showCmd=SW_NORMAL; //nCmdShow;
	place.flags=WPF_SETMINPOSITION;
	place.ptMinPosition=CPoint(0,0);
	CRect client;
	AfxGetMainWnd()->GetClientRect (client);
	place.ptMaxPosition=CPoint(client.left, client.bottom); 
//	place.ptMaxPosition=CPoint(-::GetSystemMetrics (SM_CXBORDER), -::GetSystemMetrics (SM_CYBORDER)); 
	place.rcNormalPosition=CRect(-1, -1, client.Width()/4*3, client.Height()/4*3);

	hal = theApp.GetHal();

}

CScanDoc::~CScanDoc()
{
	int i;
	for (i=0; i<axis.GetSize(); i++) delete   axis[i];
	for (i=0; i<par.GetSize();  i++) delete [] par[i];
	for (i=0; i<err.GetSize();  i++) delete [] err[i];
	for (i=0; i<chi2.GetSize(); i++) delete [] chi2[i];
	for (i=0; i<numPoints; i++) {
		delete dataArray[i];
	}
	delete [] badChannel;

}

// CScanDoc serialization
Module * CScanDoc::GetModule() {
	return module;
}

void CScanDoc::Serialize(CArchive& ar)
{
  int i, j, numBad;
  CString buf;
  char line[100], name[100];
  float version=1.1;

	if (ar.IsStoring())	{
    // storing code here:
		buf.Format ("# ALDAS %5.1f\n", VERSION);
		ar.WriteString(buf);
		buf.Format ("$%s\n", heading);
		ar.WriteString(buf);
		if (dataArray[0] != NULL)
			buf.Format ("%i %i %i\n", link, dataArray[0]->GetTriggers(), 
				dataArray[0]->GetNumChannels());
		else {
			TRACE ("CScanDoc::Serialize: dataArray[0] == NULL\n");
			return;			// FIXME: serialization has not completed 
		}                 //        so an error flag should be set.
		ar.WriteString (buf);	
		buf.Format ("%i numPoints\n", numPoints);
		ar.WriteString (buf);

		for (i=0; i<numPoints; i++) {
			dataArray[i]->Serialize(ar);
		}
		buf.Format ("%i numParam\n", numParam);
		ar.WriteString (buf);
		buf.Format ("%i numChi2\n", numChi2);
		ar.WriteString (buf);
		for (i=0; i<numChi2; i++){
			for (j=0; j<numChannels; j++) {
				buf.Format ("%f\n", chi2[i][j]);
				ar.WriteString(buf);
			}
		}
		for (i=0; i<numParam; i++) { // loop over parameters
			buf.Format ("$%s\n", axis[i]->GetLabel());
			ar.WriteString(buf);
			for (j=0; j<numChannels; j++) {
				buf.Format ("%f\t%f\n", par[i][j], err[i][j]);
				ar.WriteString(buf);
			}
		}
		if (VERSION>=1.2) {
			buf.Format ("$%s\n", moduleName);
			ar.WriteString(buf);
		}
				// count # of bad channels:
		numBad=0;
		for (i=0; i< numChannels; i++) 
			if (badChannel[i]) numBad++;
		sprintf (line, "%i bad channels\n", numBad);
		ar.WriteString (line);

		for (i=0; i< numChannels; i++) {
			if (badChannel[i]) {
				sprintf (line, "%i is bad\n", i);
				ar.WriteString (line);
			}
		}


	} 	else {   //loading code here		
		ar.ReadString(line, 100);
		if (line[0]=='#') { // version 
			sscanf (line, "# ALDAS %f", &version);
			ar.ReadString(line, 100);
		}
		heading=&(line[1]); // skip '$'

		ar.ReadString (line, 100);
		sscanf(line, "%i %i %i", &link, &trig, &numChannels);

		// delete old data points:
		for (i=0; i<numPoints; i++) {
			delete dataArray[i];
		}

		ar.ReadString (line, 100);
		sscanf(line, "%i", &numPoints);
		dataArray.SetSize(numPoints);
		for (i=0; i<numPoints; i++) {
			dataArray[i] = new CData(NULL, link);
			dataArray[i]->Serialize(ar);
		}
		ar.ReadString (line, 100);
		sscanf(line, "%i", &numParam);

		// delete old results:
		for (i=0; i<axis.GetSize(); i++) delete   axis[i];
		for (i=0; i<par.GetSize();  i++) delete [] par[i];
		for (i=0; i<err.GetSize();  i++) delete [] err[i];
		for (i=0; i<chi2.GetSize(); i++) delete [] chi2[i];
		
		if (version>=1.2) {
			ar.ReadString (line, 100);
			sscanf(line, "%i", &numChi2);
		} else {
			numChi2 = 1;
		}
		par.SetSize (numParam);
		err.SetSize (numParam);
		axis.SetSize (numParam);
		chi2.SetSize (numChi2);

		for (i=0; i<numParam; i++) {
			par[i] = new float [numChannels];
			err[i] = new float [numChannels];
		}
		for (i=0; i<numChi2; i++){
			chi2[i] = new float [numChannels];
			for (j=0; j<numChannels; j++) {
				ar.ReadString (line, 100);
				sscanf(line, "%f", &(chi2[i][j]));
			}
		}
		for (i=0; i<numParam; i++) {
			ar.ReadString (name, 100);
			for (j=0; j<numChannels; j++) {
				ar.ReadString (line, 100);
				sscanf(line, "%f %f", &(par[i][j]), &(err[i][j]));
			}
			axis[i]= new CAxis(&(name[1]), numChannels, par[i], err[i], chi2[i/(numParam/numChi2)]);
		}
		if (version>=1.2) {
			ar.ReadString (line, 100);
			moduleName=&(line[1]); // skip '$'
		} else {
			moduleName.Empty();
		}
		badChannel=new BOOL [numChannels];
		if (version>=1.2) {
			ar.ReadString (line, 80);
			sscanf (line, "%i", &numBad);
			for (i=0; i< numChannels; i++) badChannel[i]=FALSE;
			for (i=0; i< numBad; i++) {
				ar.ReadString (line, 80);
				sscanf (line, "%i", &j);
				badChannel[j]=TRUE;
			}
		}
	}
}

// CScanDoc diagnostics

#ifdef _DEBUG
void CScanDoc::AssertValid() const
{
	CDocument::AssertValid();
}

void CScanDoc::Dump(CDumpContext& dc) const
{
	CDocument::Dump(dc);
}
#endif //_DEBUG

// CScanDoc commands
