// CScanView.cpp : implementation of the CScanView class
//

#include "stdafx.h"
#include <afxtempl.h>
#include "ALDAS.h"
#include "MainFrm.h"
#include <math.h>

#include "PlaceDoc.h"
#include "ScanDoc.h"
#include "Axis.h"
#include "ScanView.h"
#include "HeadingDlg.h"
#include "NoiseFitDlg.h"
#include "Data.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

// CScanView

IMPLEMENT_DYNCREATE(CScanView, CScrollView)

BEGIN_MESSAGE_MAP(CScanView, CScrollView)
	//{{AFX_MSG_MAP(CScanView)
	ON_COMMAND(ID_HISTOGRAM_VIEW, OnHistogramView)
	ON_COMMAND(ID_LINE_FIT_VIEW, OnLineFitView)
	ON_UPDATE_COMMAND_UI(ID_HISTOGRAM_VIEW, OnUpdateHistogramView)
	ON_UPDATE_COMMAND_UI(ID_LINE_FIT_VIEW, OnUpdateLineFitView)
	ON_COMMAND(ID_SETHEADING, OnSetheading)
	ON_WM_MOUSEMOVE()
//	ON_WM_LBUTTONDOWN()
	//}}AFX_MSG_MAP
	// Standard printing commands
	ON_COMMAND(ID_FILE_PRINT, CView::OnFilePrint)
	ON_COMMAND(ID_FILE_PRINT_DIRECT, CView::OnFilePrint)
	ON_COMMAND(ID_FILE_PRINT_PREVIEW, CView::OnFilePrintPreview)
END_MESSAGE_MAP()

// CScanView construction/destruction

CScanView::CScanView()
{
	// TODO: add construction code here
  printRect.SetRect(0, 0, 200, 300);
  viewMode = LineFitView;
}

CScanView::~CScanView()
{
}

BOOL CScanView::PreCreateWindow(CREATESTRUCT& cs)
{
	// TODO: Modify the Window class or styles here by modifying
	//  the CREATESTRUCT cs

	return CView::PreCreateWindow(cs);
}

// CScanView drawing

void CScanView::OnDraw(CDC* pDC)
{
	int i;
	CScanDoc* pDoc = GetDocument();
	ASSERT_VALID(pDoc);

	CRect client, rect, page;
	CSize size; 
	CString buf;
	CData *data;
	int len = pDoc->numPoints;
	CFont *oldFont, titleFont, numberFont, smallFont;

	if (!pDC->IsPrinting()) {
		pDC->GetWindow()->GetClientRect(client);
		page=client;
      page.bottom=int(page.Width()/8.5*11.0);
		SetScrollSizes(MM_TEXT, page.Size());
		page.left=page.Width()/40;
	} else {
		 page=printRect;
		 page.left=page.right/8;
		 page.right-=page.right/12;
		 page.bottom-=page.Height()/10;
		 page.top+=page.Height()/15;
	}

// TITLE:
	titleFont.CreateFont(24*page.Width()/480, 0, 0, 0, FW_BOLD, FALSE, TRUE, 0, 
					ANSI_CHARSET, OUT_DEFAULT_PRECIS, CLIP_DEFAULT_PRECIS, 
					DEFAULT_QUALITY, DEFAULT_PITCH | FF_SWISS, "Arial");
	oldFont = (CFont*) pDC->SelectObject (&titleFont);
	size=pDC->GetTextExtent(pDoc->heading);
	pDC->TextOut(page.Width()/2-size.cx/2+page.left, 
               int(0.25*size.cy)+page.top, pDoc->heading);
	page.top += int(1.5*size.cy);

// "fine print":
	smallFont.CreateFont(12*page.Width()/480, 0, 0, 0, FW_NORMAL, FALSE, FALSE, 0,
					ANSI_CHARSET, OUT_DEFAULT_PRECIS, CLIP_DEFAULT_PRECIS, 
					DEFAULT_QUALITY, DEFAULT_PITCH | FF_SWISS, "Arial");
	pDC->SelectObject (&smallFont);
	data=pDoc->dataArray[0];
	buf.Format ("Module name: '%s',   Control block:'%s',   Strobe delay: %3i.", 
		pDoc->moduleName, data->GetCBName(), data->GetStrobeDelay());
	size=pDC->GetTextExtent(buf);
	pDC->TextOut(page.Width()/2-size.cx/2+page.left, page.bottom-int(3.25*size.cy), buf);
	buf.Format ("V_thr: %5.3f V,   V_att: %5.3f V,   V_ref: %5.3f V,   V_cal: %5.3f V", 
		data->GetLLDacVoltage(data->GetLink()), data->GetDSPDacVoltage(0), 
		data->GetDSPDacVoltage(1), data->GetDSPDacVoltage(3));
	size=pDC->GetTextExtent(buf);
	pDC->TextOut(page.Width()/2-size.cx/2+page.left, page.bottom-int(2.25*size.cy), buf);
	buf.Format ("Bias Voltage: %5.0f V,   Alignment delay: %4i.", 
		data->GetBiasVoltage(), data->GetAlignmentDelay());
	size=pDC->GetTextExtent(buf);
	pDC->TextOut(page.Width()/2-size.cx/2+page.left, page.bottom-int(1.25*size.cy), buf);
	page.bottom -= int(5*size.cy);



//	graphs:
	if (pDoc->numParam>0) {
		rect.SetRect(page.left, page.top, page.right, page.top+page.Height()/pDoc->numParam);
		pDC->SelectObject (oldFont);
		numberFont.CreateFont(14*rect.Width()/480, 0, 0, 0, FW_NORMAL, FALSE, FALSE, 0,
						ANSI_CHARSET, OUT_DEFAULT_PRECIS, CLIP_DEFAULT_PRECIS, 
						DEFAULT_QUALITY, DEFAULT_PITCH | FF_SWISS, "Arial");
		oldFont = (CFont*) pDC->SelectObject (&numberFont);


		if (viewMode == LineFitView) {
			for (i=0; i<pDoc->numParam; i++){
				pDoc->axis[i]->DrawData (pDC, rect);
				rect.OffsetRect (0, rect.Height());
			}
		} else if (viewMode == HistogramView) {
			for (i=0; i<pDoc->numParam; i++){
				pDoc->axis[i]->DrawHistogram (pDC, rect);
				rect.OffsetRect (0, rect.Height());
			}
		}
	}
	pDC->SelectObject (oldFont);
	rect.SetRect(page.left, page.top, page.right, page.top+page.Height()*0.1);
}

// CScanView printing

BOOL CScanView::OnPreparePrinting(CPrintInfo* pInfo)
{
	// default preparation
	return DoPreparePrinting(pInfo);
}

void CScanView::OnBeginPrinting(CDC* /*pDC*/, CPrintInfo* /*pInfo*/)
{
	// TODO: add extra initialization before printing
}

void CScanView::OnEndPrinting(CDC* /*pDC*/, CPrintInfo* /*pInfo*/)
{
	// TODO: add cleanup after printing
}


void CScanView::OnInitialUpdate()
{
	CScanDoc* pDoc = GetDocument();
	ASSERT_VALID(pDoc);
  
  CScrollView::OnInitialUpdate();
	CSize sizeTotal;
	// TODO: calculate the total size of this view
	sizeTotal.cx = 480;
	sizeTotal.cy = 800;
	SetScrollSizes(MM_TEXT, sizeTotal);
	TRACE("In OnInitialUpdate()\n");
}

void CScanView::OnUpdate(CView* pSender, LPARAM lHint, CObject* pHint) {
	CSize size = GetTotalSize();
	TRACE("xSize = %i, ySize = %i\n", size.cx, size.cy);
//	CView::OnUpdate(CView* pSender, LPARAM lHint, CObject* pHint);
}

// CScanView diagnostics

#ifdef _DEBUG
void CScanView::AssertValid() const
{
	CView::AssertValid();
}

void CScanView::Dump(CDumpContext& dc) const
{
	CView::Dump(dc);
}

CScanDoc* CScanView::GetDocument() // non-debug version is inline
{
//	ASSERT(m_pDocument->IsKindOf(RUNTIME_CLASS(CScanDoc)));
	return (CScanDoc*)m_pDocument;
}
#endif //_DEBUG

// CScanView message handlers




void CScanView::OnSetheading() 
{
	CScanDoc* pDoc = GetDocument();
	ASSERT_VALID(pDoc);
	CHeadingDlg dlg;

	dlg.heading=pDoc->heading;
	if (dlg.DoModal()!=IDOK) return;
	pDoc->heading=dlg.heading;
	pDoc->SetModifiedFlag(TRUE);
	pDoc->UpdateAllViews(NULL);
}


void CScanView::OnPrint(CDC* pDC, CPrintInfo* pInfo) 
{
	printRect=pInfo->m_rectDraw;
	CScrollView::OnPrint(pDC, pInfo);
}


void CScanView::OnMouseMove(UINT nFlags, CPoint point) 
{
	CString str;
	char pm=177; // plus/minus char
	char sq=178; // square char
	int i;		// channel number
					// true if cursor is inside the graph rectangle.
			 
	CMainFrame* pFrame = (CMainFrame*) AfxGetApp()->m_pMainWnd;
	CStatusBar* pStatus= &pFrame->m_wndStatusBar;
	
	CScanDoc* pDoc = GetDocument();
	CDC * thisDC = GetDC();
	CPoint scroll = GetScrollPosition( );
	point=point+scroll; // add scroll offset to convert from window to document
	if (pStatus) {
		if (viewMode == LineFitView) {
			i=pDoc->axis[0]->GetChannelNumber(point);
			if (i>=0 && (pDoc->axis[0]->CursorInsideGraphRect(point) ||
				pDoc->axis[1]->CursorInsideGraphRect(point))) {
				str.Format ("Channel %3i: %s = %f %c %f,  %s = %f %c %f, chi%c = %f", i, 
								pDoc->axis[0]->GetLabel(), pDoc->par[0][i], pm, pDoc->err[0][i], 
								pDoc->axis[1]->GetLabel(), pDoc->par[1][i], pm, pDoc->err[1][i],
								sq, pDoc->chi2[0][i]);
			} else {
				str.Empty();
			}
			pStatus->SetPaneText (0, str);
		}
	}
	CScrollView::OnMouseMove(nFlags, point);
}


void CScanView::OnHistogramView() 
{
	// TODO: Add your command handler code here
	viewMode = HistogramView;
	Invalidate();
	SendMessage(WM_PAINT);
}

void CScanView::OnLineFitView() 
{
	// TODO: Add your command handler code here
	viewMode = LineFitView;
	Invalidate();
	SendMessage(WM_PAINT);
}

void CScanView::OnUpdateHistogramView(CCmdUI* pCmdUI) 
{
	pCmdUI->SetCheck(viewMode == HistogramView);
}

void CScanView::OnUpdateLineFitView(CCmdUI* pCmdUI) 
{
	pCmdUI->SetCheck(viewMode == LineFitView);
}
