// StrobeScanDoc.cpp : implementation of the CCollectDataDoc class
//

#include "stdafx.h"
#include <afxtempl.h>
#include <math.h>
#include "ALDAS.h"
#include "MainFrm.h"

#include "PlaceDoc.h"
#include "ScanDoc.h"
#include "Axis.h"
#include "StrobeScanDoc.h"
#include "StrobeDlg.h"
#include "TakeDataDlg.h"
#include "Data.h"
#include "Message.h"
#include "DSPcard.h"
#include "llcard.h"
#include "module.h"

#include "hal.h"

#include "SelectModuleDlg.h"
#include "TriggerSourceDlg.h"

#include "fitLSQ.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

extern CALDASApp theApp;

// CCollectDataDoc

IMPLEMENT_DYNCREATE(CStrobeScanDoc, CDocument)

BEGIN_MESSAGE_MAP(CStrobeScanDoc, CDocument)
	//{{AFX_MSG_MAP(CStrobeScanDoc)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

// CCollectDataDoc construction/destruction

CStrobeScanDoc::CStrobeScanDoc()
{
	numParam = 2;
	axis.SetSize(2);
	par.SetSize(2);
	err.SetSize(2);
	chi2.SetSize(1);
}

CStrobeScanDoc::~CStrobeScanDoc()
{
}

BOOL CStrobeScanDoc::OnNewDocument()
{
	int i, j;
	UINT triggers;
	CString cbOld; // name of old control block # 3
	CMainFrame* pFrame = (CMainFrame*) AfxGetApp()->m_pMainWnd;
	CStatusBar* pStatus= &pFrame->m_wndStatusBar;
	CString buf;

	if (!CDocument::OnNewDocument())
		return FALSE;
	
	CSelectModuleDlg dlg;
	if (dlg.DoModal() != IDOK) return FALSE;
	int moduleIndex = dlg.moduleNumber;

	module = hal->GetModule(moduleIndex);
	moduleName = module->GetModuleName();

	hal->ModifySettings(moduleIndex);

	CTriggerSourceDlg tdlg;
	if (tdlg.DoModal() != IDOK) return FALSE;
	triggers = tdlg.triggers;

	///////////////// Get min, max, and increment. //////////////////
	CStrobeDlg strobeDlg;
	if (strobeDlg.DoModal() != IDOK) return FALSE;


	increment = strobeDlg.increment;
	max = strobeDlg.maximum;
	min = strobeDlg.minimum;
	calLine[0]=strobeDlg.line0;
	calLine[1]=strobeDlg.line1;
	calLine[2]=strobeDlg.line2;
	calLine[3]=strobeDlg.line3;
	triggers = strobeDlg.triggers;
	numPoints = (UINT) ((max - min)/increment + 1.5);
	dataArray.SetSize(numPoints);

	numChannels=module->GetNumChannels();
	if (numChannels<1) return FALSE;
	par[0] = new float [numChannels];
	par[1] = new float [numChannels];
	err[0] = new float [numChannels];
	err[1] = new float [numChannels];
	chi2[0] = new float [numChannels];
	badChannel=new BOOL [numChannels];
	for (i=0; i<numChannels; i++) badChannel[i]=module->badChannel[i];


	CTakeDataDlg take;
	take.maxTriggers = triggers;

	int oldDelay=hal->GetStrobeDelay(moduleIndex);
	int curDelay=min;
	for (j=0; j<numPoints; j++) {
		hal->SetStrobeDelay(moduleIndex, curDelay);
		dataArray[j] = new CData (moduleIndex);
		buf.Format("Strobe delay scan %i of %i: Delay = %i\n", j+1, numPoints, curDelay);
		pStatus->SetPaneText (0, buf);
		take.Init(dataArray[j], moduleIndex, triggers, tdlg.triggerSource);
		if (take.DoModal()!=IDOK)  {
			hal->SetStrobeDelay(moduleIndex, oldDelay);
			pStatus->SetPaneText (0, "");
			return FALSE; // canceled
		}
		curDelay += increment;
	}

	pStatus->SetPaneText (0, "");
	hal->SetStrobeDelay (moduleIndex, oldDelay);
	AnalyzeData();

	heading.Format("Strobe Delay Scan of '%s', %d Triggers", 
						module->GetModuleName(), dataArray[0]->GetTriggers());

	SetModifiedFlag(TRUE);
	return TRUE;
}


void CStrobeScanDoc::AnalyzeData() {
	int i, j;
	float curDelay = min;
//	CCurveFitter linefit;
	float *tempY, *tempX, *sig;
	UINT hits = 0;
	float ave_eff, a[4], m1, m2;
	int trigs = 0, j1, j2;
//	float q, b, e1, e2, chi;
	int nonzero = 0; // count # of points with >1 hits
	int lista[4] = {0, 1, 2, 3};
	float lamda, oldChi2;
	int nfit;
	void plateau (float x, float a[], float * y, float dyda[], int na);
	float **alpha, **covar;

	
	tempY = new float [numPoints];
	tempX = new float [numPoints];
	sig = new float [numPoints];
	
	covar=matrix(1, 3, 1, 3);
	alpha=matrix(1, 3, 1, 3);

	for (i=0; i<numChannels; i++) {
		par[0][i] = 0;
		par[1][i] = 0;
		if (module->badChannel[i]) {
			chi2[0][i] = -2;
		} else {
			nonzero = 0;
			curDelay = min;
			for (j=0; j<numPoints; j++) {
				trigs = dataArray[j]->GetTriggers();
				hits = dataArray[j]->hits[i];
				ave_eff = float (hits)/float(trigs);
				if (ave_eff > 0.5) nonzero++;
				tempY[j] = ave_eff;
				sig[j] = sqrt(ave_eff*(1.0-ave_eff)/trigs)/ave_eff; // Binomial stat.
				if (sig[j]==0)	sig[j]=0.1; 
				tempX[j] = curDelay;
				curDelay += increment;
				if (curDelay>255) curDelay=255;
			}
			if ((nonzero>1)&&(calLine[i%4])) { // more than 1 non-zero efficiency
				j1=0;
				while ((j1<numPoints)&&(tempY[j1]<0.5)) j1++;
				j2=j1;
				while ((j2<numPoints)&&(tempY[j2]>0.5)) j2++;
				
				if (j1==0){
					a[1]=tempX[j1];
					m1=20;
				} else { // linear interpolation
					m1=(tempY[j1]-tempY[j1-1])/(tempX[j1]-tempX[j1-1]);
					a[1]=tempX[j1-1]+(0.5-tempY[j1-1])/m1;
					if (a[1]<-30) a[1]=-30;
					if (a[1]>300) a[1]=300;
				}
				if (j2==0){
					a[2]=tempX[j2];
					m2=20;
				} else { // linear interpolation
					m2=(tempY[j2]-tempY[j2-1])/(tempX[j2]-tempX[j2-1]);
					a[2]=tempX[j2-1]+(0.5-tempY[j2-1])/m2;
					if (a[2]<-30) a[2]=-30;
					if (a[2]>300) a[2]=300;
				}
				a[3]=1.0/sqrt(2.0*3.141592)/fabs(0.5*(m1-m2));
				if (a[3]<10.0) a[3] = 10.0;
				if (a[3]>40.0) a[3] = 40.0;
				if (a[2] <= a[1]) {
					TRACE ("a1=%f, a2=%f, a3=%f, j2=%i\n", a[1], a[2], a[3], j2);
					a[2]=a[1]+10.0;
				}
				lamda = -1.0;
				nfit=0;
				chi2[0][i]=1E10;
				do {
					oldChi2=chi2[0][i];
					mrqmin(tempX-1, tempY-1, sig-1, numPoints, a, 3, lista, 3, covar,
							 alpha, &(chi2[0][i]), &plateau, &lamda);
//					TRACE ("mrqmin: nfit = %i, lamda=%f, a1=%f, a2=%f, a3=%f, chi2 = %f\n", 
//						nfit, lamda, a[1], a[2], a[3], chi2[i]);
					nfit++;
				} while ((nfit<2)||((nfit<10)&&(oldChi2-chi2[0][i]>0.1)));
				// calculate cov matrix:
				lamda = 0.0;
				mrqmin(tempX-1, tempY-1, sig-1, numPoints, a, 3, lista, 3, covar,
						 alpha, &(chi2[0][i]), &fintgauss, &lamda);

				par[0][i] = a[1];
				par[1][i] = a[2];
				err[0][i] = sqrt(covar[1][1]);
				err[1][i] = sqrt(covar[2][2]);
			} else { // not ((nonzero>1)&&(calLine[i%4]))
				par[0][i] = 128;
				par[1][i] = 128;
				err[0][i] = 0;
				err[1][i] = 0;
				chi2[0][i] = -1;
			}
		}
	}
	free_matrix (covar, 1, 3, 1, 3);
	free_matrix (alpha, 1, 3, 1, 3);

	delete [] sig;
	delete [] tempY;
	delete [] tempX;

	axis[0]=new CAxis ("Start point", numChannels, par[0], err[0], chi2[0]);
	axis[1]=new CAxis ("End point", numChannels, par[1], err[1], chi2[0]);
}


/*void CStrobeScanDoc::Serialize(CArchive& ar) {
	CString buf;
//	char line[100];
	if (ar.IsStoring())	{
		CScanDoc::Serialize(ar);
	} else {
		CScanDoc::Serialize(ar);
	}
}*/


// CStrobeScanDoc diagnostics

#ifdef _DEBUG
void CStrobeScanDoc::AssertValid() const
{
	CDocument::AssertValid();
}

void CStrobeScanDoc::Dump(CDumpContext& dc) const
{
	CDocument::Dump(dc);
}
#endif //_DEBUG