// CollectDataDoc.cpp : implementation of the CCollectDataDoc class
//

#include "stdafx.h"
#include <afxtempl.h>
#include <math.h>
#include "ALDAS.h"
#include "MainFrm.h"

#include "PlaceDoc.h"
#include "ScanDoc.h"
#include "Axis.h"
#include "ThresholdScanDoc.h"
#include "SelectDspDlg.h"
#include "DSPDlg.h"
#include "ThresholdDlg.h"
#include "TakeDataDlg.h"
#include "Data.h"
#include "Message.h"
#include "DSPcard.h"
#include "llcard.h"
//#include "Hybrid.h"
#include "module.h"

#include "hal.h"

#include "SelectModuleDlg.h"
#include "TriggerSourceDlg.h"

//#include "CurveFitter.h"
#include "fitLSQ.h"

extern CALDASApp theApp;


#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

// CCollectDataDoc

IMPLEMENT_DYNCREATE(CThresholdScanDoc, CDocument)

BEGIN_MESSAGE_MAP(CThresholdScanDoc, CDocument)
	//{{AFX_MSG_MAP(CThresholdScanDoc)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

// CCollectDataDoc construction/destruction

CThresholdScanDoc::CThresholdScanDoc()
{
	numParam = 2;
	axis.SetSize(2);
	par.SetSize(2);
	err.SetSize(2);
	chi2.SetSize(1);
}

CThresholdScanDoc::~CThresholdScanDoc()
{
}

BOOL CThresholdScanDoc::OnNewDocument()
{
	int i, k;
	int triggers;
	CString cbOld; // name of old control block # 3
	CString buf;
	CMainFrame* pFrame = (CMainFrame*) AfxGetApp()->m_pMainWnd;
	CStatusBar* pStatus= &pFrame->m_wndStatusBar;

	if (!CDocument::OnNewDocument())
		return FALSE;

	CSelectModuleDlg dlg;
	if (dlg.DoModal() != IDOK) return FALSE;
	int moduleIndex = dlg.moduleNumber;

	hal->ModifySettings(moduleIndex);

	CTriggerSourceDlg tdlg;
	if (tdlg.DoModal() != IDOK) return FALSE;
	triggers = tdlg.triggers;

	///////////////// Get min, max, and increment. //////////////////
	CThresholdDlg threshDlg;
	if (threshDlg.DoModal() != IDOK)
		return FALSE;


	increment = threshDlg.increment;
	max = threshDlg.maximum;
	min = threshDlg.minimum;
	calLine[0] = threshDlg.line0;
	calLine[1] = threshDlg.line1;
	calLine[2] = threshDlg.line2;
	calLine[3] = threshDlg.line3;
//	triggers = threshDlg.triggers;
	numPoints = (UINT) ((max - min)/increment + 1.5);
	dataArray.SetSize(numPoints);

	hal->SetCalibration(moduleIndex, threshDlg.calVoltage);
	numChannels=hal->GetNumChannels(moduleIndex);
	if (numChannels<1) return FALSE;

	module = hal->GetModule(moduleIndex);
	moduleName = module->GetModuleName();

	if (numChannels<1) return FALSE;
	par[0] = new float [numChannels];
	par[1] = new float [numChannels];
	err[0] = new float [numChannels];
	err[1] = new float [numChannels];
	chi2[0] = new float [numChannels];
	badChannel=new BOOL [numChannels];
	for (i=0; i<numChannels; i++) badChannel[i]=module->badChannel[i];

	///////////////// Take the data /////////////////////////////////
	CTakeDataDlg take;
	take.maxTriggers = triggers;

	float oldThreshold;
	oldThreshold = hal->GetThreshold(moduleIndex);

	float curThreshold = min;
	dataArray.SetSize(numPoints);

	for (int j=0; j<numPoints; j++) {
		hal->SetThreshold(moduleIndex, curThreshold);
		dataArray[j] = new CData (moduleIndex);
		buf.Format("Threshold scan %i of %i: Threshold = %f V\n", j+1, numPoints, curThreshold);
		pStatus->SetPaneText (0, buf);
		take.Init(dataArray[j], moduleIndex, triggers, tdlg.triggerSource);
		if (take.DoModal()!=IDOK)  {
			pStatus->SetPaneText (0, "");
			hal->SetThreshold(moduleIndex, oldThreshold);
			return FALSE; // canceled
		}
		curThreshold += increment;
	}

	pStatus->SetPaneText (0, "");
	hal->SetThreshold(moduleIndex, oldThreshold); 

	AnalyzeData();

	heading.Format("Threshold Scan of '%s', %d Triggers", 
						module->GetModuleName(), dataArray[0]->GetTriggers());

	
	SetModifiedFlag(TRUE);
	return TRUE;
}


// We cut the fitting function for each channel based on three criteria:
// 1. If the channel is marked as bad for some reason.
// 2. If the last data point has a higher efficiency then first data point.
// 3. If there are no non-zero efficiency data points.
void CThresholdScanDoc::AnalyzeData() {
	int i, j;
	float curThreshold = min;
	float *tempY, *tempX, *sig;
	UINT hits = 0;
	float ave_eff, a[3], m1;
	int trigs = 0, j1;
	int nonzero; // count # of points with >1 hits
	int lista[3] = {0, 1, 2};
	float lamda, oldChi2;
	int nfit;
	void fintgauss (float, float*, float*, float*, int);
	float **alpha, **covar;


	// tempY = average efficiency for a channel at each threshold value.
	// tempX = the threshold
	tempY = new float [numPoints];
	tempX = new float [numPoints];
	sig = new float [numPoints];
	
	covar=matrix(1, 2, 1, 2);
	alpha=matrix(1, 2, 1, 2);

	for (i=0; i<numChannels; i++) {
		par[0][i] = 0;
		par[1][i] = 0;
		err[0][i] = 0;
		err[1][i] = 0;

		// If the channel is marked bad, don't fit a curve to it.
		if (badChannel[i]) {
			chi2[0][i] = -2;
			continue;
		}

		nonzero = 0;
		curThreshold = min;
		for (j=0; j<numPoints; j++) {
			trigs = dataArray[j]->GetTriggers();
			hits = dataArray[j]->hits[i];
			ave_eff = float (hits)/float(trigs);
			if (ave_eff > 0.001) nonzero++;
			tempY[j] = ave_eff;
			sig[j] = sqrt(ave_eff*(1.0-ave_eff)/trigs)/ave_eff; // Binomial stat.
			if (sig[j]<0.01)	sig[j]=0.01; 
			tempX[j] = curThreshold;
			curThreshold += increment;
		}

		// If the data for this channel looks bad, don't fit.
		if (tempY[0]<tempY[numPoints-1]) {
			chi2[0][i] = -3;
			continue;
		}

		// Find the 50% point
		j1=0;
		while ((j1<numPoints)&&(tempY[j1]>0.5)) j1++;

		// If there are no non-zero efficiency data points, don't fit.
		if ((nonzero>1)&&(calLine[i%4])&&(j1<numPoints)) { // more than 1 non-zero efficiency
				
			if (j1==0){
				a[1]=tempX[j1];
				m1=20;
			} else { // linear interpolation
				m1=(tempY[j1]-tempY[j1-1])/(tempX[j1]-tempX[j1-1]);
				a[1]=tempX[j1-1]+(0.5-tempY[j1-1])/m1;
				if (a[1]<-30) a[1]=-30;
				if (a[1]>300) a[1]=300;
			}
			a[2]=1.0/sqrt(2.0*3.141592)/fabs(m1);
			if (a[2]<0.0) a[2] = 0.0;
			if (a[2]>4.0) a[2] = 4.0;
//			TRACE ("a1=%f, a2=%f, j1=%i\n", a[1], a[2], j1);

			lamda = -1.0;
			nfit=0;
			chi2[0][i]=1E10;
			do {
				oldChi2=chi2[0][i];
				mrqmin(tempX-1, tempY-1, sig-1, numPoints, a, 2, lista, 2, covar,
						 alpha, &(chi2[0][i]), &fintgauss, &lamda);
//				TRACE ("mrqmin: nfit = %i, lamda=%f, a1=%f, a2=%f, chi2 = %f\n", 
//						nfit, lamda, a[1], a[2], chi2[i]);
				nfit++;
			} while ((nfit<2)||((nfit<10)&&(oldChi2-chi2[0][i]>0.1)));
			// calculate cov matrix:
			lamda = 0.0;
			mrqmin(tempX-1, tempY-1, sig-1, numPoints, a, 2, lista, 2, covar,
					 alpha, &(chi2[0][i]), &fintgauss, &lamda);

			par[0][i] = a[1];
			par[1][i] = a[2];
			err[0][i] = sqrt(covar[1][1]);
			err[1][i] = sqrt(covar[2][2]);
		} else { // not ((nonzero>1)&&(calLine[i%4]))
			par[0][i] = 0.5*(min+max);
			par[1][i] = (max-min)/numPoints;
			err[0][i] = 0;
			err[1][i] = 0;
			chi2[0][i] = -1;
		}
	}
	free_matrix (covar, 1, 2, 1, 2);
	free_matrix (alpha, 1, 2, 1, 2);

	delete [] tempY;
	delete [] tempX;
	delete [] sig;

	axis[0]=new CAxis ("Response", numChannels, par[0], err[0], chi2[0]);
	axis[1]=new CAxis ("Sigma", numChannels, par[1], err[1], chi2[0]);
}




// CThresholdScanDoc serialization

/*void CThresholdScanDoc::Serialize(CArchive& ar)
{
  CString buf;
//  char line[100];
	if (ar.IsStoring())	{
		CScanDoc::Serialize(ar);
	} else {
		CScanDoc::Serialize(ar);
	}
}*/


// CThresholdScanDoc diagnostics

#ifdef _DEBUG
void CThresholdScanDoc::AssertValid() const
{
	CDocument::AssertValid();
}

void CThresholdScanDoc::Dump(CDumpContext& dc) const
{
	CDocument::Dump(dc);
}
#endif //_DEBUG