// CCalibrationScanView.cpp : implementation of the CCalibrationScanView class
//

#include "stdafx.h"
#include <afxtempl.h>
#include "ALDAS.h"
#include "MainFrm.h"
#include <math.h>

#include "PlaceDoc.h"
#include "ScanDoc.h"
#include "Axis.h"
#include "CalibrationScanDoc.h"
#include "CalibrationScanView.h"
#include "HeadingDlg.h"
#include "Data.h"
#include "FitResultDlg.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

// CCalibrationScanView

IMPLEMENT_DYNCREATE(CCalibrationScanView, CScrollView)

BEGIN_MESSAGE_MAP(CCalibrationScanView, CScrollView)
	//{{AFX_MSG_MAP(CCalibrationScanView)
	ON_COMMAND(ID_SETHEADING, OnSetheading)
	ON_WM_MOUSEMOVE()
	ON_WM_LBUTTONDOWN()
	//}}AFX_MSG_MAP
	// Standard printing commands
	ON_COMMAND(ID_FILE_PRINT, CView::OnFilePrint)
	ON_COMMAND(ID_FILE_PRINT_DIRECT, CView::OnFilePrint)
	ON_COMMAND(ID_FILE_PRINT_PREVIEW, CView::OnFilePrintPreview)
END_MESSAGE_MAP()

// CCalibrationScanView construction/destruction

CCalibrationScanView::CCalibrationScanView()
{
	// TODO: add construction code here
  printRect.SetRect(0, 0, 200, 300);
}

CCalibrationScanView::~CCalibrationScanView()
{
}

BOOL CCalibrationScanView::PreCreateWindow(CREATESTRUCT& cs)
{
	// TODO: Modify the Window class or styles here by modifying
	//  the CREATESTRUCT cs

	return CView::PreCreateWindow(cs);
}

// CCalibrationScanView drawing

void CCalibrationScanView::OnDraw(CDC* pDC)
{
	int i;
	CPen *oldPen, newPen; // newPen is destroyed automatically.
	CCalibrationScanDoc* pDoc = GetDocument();
	ASSERT_VALID(pDoc);

	CRect client, rect, page;
	CSize size; 
	CString buf;
	CData *data;
	int len = pDoc->numPoints;
	CFont *oldFont, titleFont, numberFont, smallFont;

	if (!pDC->IsPrinting()) {
		pDC->GetWindow()->GetClientRect(client);
		page=client;
      page.bottom=int(page.Width()/8.5*11.0);
		SetScrollSizes(MM_TEXT, page.Size());
		page.left=page.Width()/40;
	} else {
		 page=printRect;
		 page.left=page.right/8;
		 page.right-=page.right/12;
		 page.bottom-=page.Height()/10;
		 page.top+=page.Height()/15;
	}

// TITLE:
	titleFont.CreateFont(24*page.Width()/480, 0, 0, 0, FW_BOLD, FALSE, TRUE, 0, 
					ANSI_CHARSET, OUT_DEFAULT_PRECIS, CLIP_DEFAULT_PRECIS, 
					DEFAULT_QUALITY, DEFAULT_PITCH | FF_SWISS, "Arial");
	oldFont = (CFont*) pDC->SelectObject (&titleFont);
	size=pDC->GetTextExtent(pDoc->heading);
	pDC->TextOut(page.Width()/2-size.cx/2+page.left, 
               int(0.25*size.cy)+page.top, pDoc->heading);
	page.top += int(1.5*size.cy);

// "fine print":
	smallFont.CreateFont(12*page.Width()/480, 0, 0, 0, FW_NORMAL, FALSE, FALSE, 0,
					ANSI_CHARSET, OUT_DEFAULT_PRECIS, CLIP_DEFAULT_PRECIS, 
					DEFAULT_QUALITY, DEFAULT_PITCH | FF_SWISS, "Arial");
	pDC->SelectObject (&smallFont);
	data=pDoc->dataArray[0];
	buf.Format ("Module name: '%s',   Control block:'%s',   Strobe delay: %3i.", 
		pDoc->moduleName, data->GetCBName(), data->GetStrobeDelay());
	size=pDC->GetTextExtent(buf);
	pDC->TextOut(page.Width()/2-size.cx/2+page.left, page.bottom-int(3.25*size.cy), buf);
	buf.Format ("V_thr: %5.3f V,   V_att: %5.3f V,   V_ref: %5.3f V,   V_cal: %5.3f V", 
		data->GetLLDacVoltage(data->GetLink()), data->GetDSPDacVoltage(0), 
		data->GetDSPDacVoltage(1), data->GetDSPDacVoltage(3));
	size=pDC->GetTextExtent(buf);
	pDC->TextOut(page.Width()/2-size.cx/2+page.left, page.bottom-int(2.25*size.cy), buf);
	buf.Format ("Bias Voltage: %5.0f V,   Alignment delay: %4i.", 
		data->GetBiasVoltage(), data->GetAlignmentDelay());
	size=pDC->GetTextExtent(buf);
	pDC->TextOut(page.Width()/2-size.cx/2+page.left, page.bottom-int(1.25*size.cy), buf);
	page.bottom -= int(5*size.cy);

	pDC->SelectObject (oldFont);
	numberFont.CreateFont(14*page.Width()/480, 0, 0, 0, FW_NORMAL, FALSE, FALSE, 0,
					ANSI_CHARSET, OUT_DEFAULT_PRECIS, CLIP_DEFAULT_PRECIS, 
					DEFAULT_QUALITY, DEFAULT_PITCH | FF_SWISS, "Arial");
	oldFont = (CFont*) pDC->SelectObject (&numberFont);
// sensitivity:
	float threshold1fC = (-pDoc->coeff[1]+sqrt(pDoc->coeff[1]*pDoc->coeff[1]-
		4.0*(pDoc->coeff[0]-1.0)*pDoc->coeff[2]))/2.0/pDoc->coeff[2];
	float sens = 1.0/(pDoc->coeff[1]+2.0*pDoc->coeff[2]*threshold1fC);
	buf.Format ("Sensitivity at 1 fC: %5.1f mV/fC", sens*1000 /* mV */);
	size=pDC->GetTextExtent(buf);
	pDC->TextOut(page.Width()/2-size.cx/2+page.left, page.bottom-int(2.5*size.cy), buf);
	buf.Format ("Threshold at 1 fC: %5.1f mV", threshold1fC*1000 /* mV */);
	size=pDC->GetTextExtent(buf);
	pDC->TextOut(page.Width()/2-size.cx/2+page.left, page.bottom-int(1.25*size.cy), buf);
	page.bottom -= int(3.5*size.cy);


//	graphs:
	if (pDoc->thrPoints>0) {
		rect.SetRect(page.left, page.top, page.right, page.top+page.Height());

		pDoc->threshold.PlotVersus (pDC, rect, pDoc->response);
	}
	pDC->SelectObject (oldFont);
	rect.SetRect(page.left, page.top, page.right, page.top+page.Height()*0.1);

	// draw parabola:
	newPen.CreatePen(PS_SOLID, 1, RGB(128, 0, 0));
	oldPen=pDC->SelectObject(&newPen);
	float range=pDoc->threshold.GetHigh()-pDoc->threshold.GetLow();
   float y=pDoc->threshold.GetLow();
   int iy=pDoc->threshold.MapYValue(y);
   float x=pDoc->coeff[0]+pDoc->coeff[1]*y+pDoc->coeff[2]*y*y;
   int ix=pDoc->response->MapXValue(x);
   pDC->MoveTo(ix, iy);
   for (i=1; i<=50; i++){
      y=pDoc->threshold.GetLow()+i*range/50.0;
	   iy=pDoc->threshold.MapYValue(y);
		x=pDoc->coeff[0]+pDoc->coeff[1]*y+pDoc->coeff[2]*y*y;
		ix=pDoc->response->MapXValue(x);
		if ((x<pDoc->response->GetHigh())&&(x>pDoc->response->GetLow())) {
			pDC->LineTo(ix, iy);
		} else {
			pDC->MoveTo(ix, iy);
		} 
	}
	pDC->SelectObject(oldPen);
}

// CCalibrationScanView printing

BOOL CCalibrationScanView::OnPreparePrinting(CPrintInfo* pInfo)
{
	// default preparation
	return DoPreparePrinting(pInfo);
}

void CCalibrationScanView::OnBeginPrinting(CDC* /*pDC*/, CPrintInfo* /*pInfo*/)
{
	// TODO: add extra initialization before printing
}

void CCalibrationScanView::OnEndPrinting(CDC* /*pDC*/, CPrintInfo* /*pInfo*/)
{
	// TODO: add cleanup after printing
}


void CCalibrationScanView::OnInitialUpdate()
{
	CCalibrationScanDoc* pDoc = GetDocument();
	ASSERT_VALID(pDoc);
  
  CScrollView::OnInitialUpdate();
	CSize sizeTotal;
	// TODO: calculate the total size of this view
	sizeTotal.cx = 480;
	sizeTotal.cy = 800;
	SetScrollSizes(MM_TEXT, sizeTotal);
}

// CCalibrationScanView diagnostics

#ifdef _DEBUG
void CCalibrationScanView::AssertValid() const
{
	CView::AssertValid();
}

void CCalibrationScanView::Dump(CDumpContext& dc) const
{
	CView::Dump(dc);
}

CCalibrationScanDoc* CCalibrationScanView::GetDocument() // non-debug version is inline
{
//	ASSERT(m_pDocument->IsKindOf(RUNTIME_CLASS(CalibrationScanDoc)));
	return (CCalibrationScanDoc*)m_pDocument;
}
#endif //_DEBUG

// CCalibrationScanView message handlers




void CCalibrationScanView::OnSetheading() 
{
	CCalibrationScanDoc* pDoc = GetDocument();
	ASSERT_VALID(pDoc);
	CHeadingDlg dlg;

	dlg.heading=pDoc->heading;
	if (dlg.DoModal()!=IDOK) return;
	pDoc->heading=dlg.heading;
	pDoc->SetModifiedFlag(TRUE);
	pDoc->UpdateAllViews(NULL);
}


void CCalibrationScanView::OnPrint(CDC* pDC, CPrintInfo* pInfo) 
{
	printRect=pInfo->m_rectDraw;
	CScrollView::OnPrint(pDC, pInfo);
}


void CCalibrationScanView::OnMouseMove(UINT nFlags, CPoint point) 
{
	CString str;
	char pm=177; // plus/minus char
	char sq=178; // square char
	int i;		// channel number
					// true if cursor is inside the graph rectangle.
			 
	CMainFrame* pFrame = (CMainFrame*) AfxGetApp()->m_pMainWnd;
	CStatusBar* pStatus= &pFrame->m_wndStatusBar;
	
	CCalibrationScanDoc* pDoc = GetDocument();
	CDC * thisDC = GetDC();
	CPoint scroll = GetScrollPosition( );
	point=point+scroll;
	if (pStatus) {
		i=pDoc->threshold.GetChannelNumberY(point);
		if (i>=0 && pDoc->threshold.CursorInsideGraphRect(point)) {
			str.Format ("Threshold = %f, Response = %f %c %f",  
							pDoc->threshold.GetData()[i],
							pDoc->mean[i], pm, pDoc->meanErr[i]);
		} else {
			str.Empty();
		}
		pStatus->SetPaneText (0, str);
	}
	CScrollView::OnMouseMove(nFlags, point);
}


void CCalibrationScanView::OnLButtonDown(UINT nFlags, CPoint point) 
{
	int iThr; // threshold number
	float triggers; 
	CCalibrationScanDoc* pDoc = GetDocument();

	CPoint scroll = GetScrollPosition( );
	point=point+scroll;

	iThr=pDoc->threshold.GetChannelNumberY(point);
	if ((iThr>=0)&& pDoc->threshold.CursorInsideGraphRect(point)) {
		// get the data to be displayed:
		
		CFitResultDlg dlg;
		dlg.Init(pDoc->axis[2*iThr+0], NULL, pDoc->mean[iThr], pDoc->meanErr[iThr]);
		if (dlg.DoModal()==IDOK) {
		//	pDoc->badChannel[iThr] = dlg.badChannel;
		}
	}
	
	CScrollView::OnLButtonDown(nFlags, point);
}
