#include "stdafx.h"
#include <afxtempl.h>

#define I_AM_HOST
#include "dsp.h"
#include "hostspec.h"
#include "eboot.h"
#include "sys_gen.h"
#include "sys_pub.h"
#include "sys_priv.h"
#include "strips.h"
#define VXIWIN32
#include "nivxi.h"

#include "message.h"
#include "llcard.h"
#include "dspcard.h"
#include "hostspec.h"
#include "bootclass.h"
#include "TccCard.h"
#include "Aldas.h"

#include "module.h"
#include "hal.h"

extern CALDASApp theApp;

IMPLEMENT_SERIAL( DSP, CObject, 1 )


DSP::DSP(int card_num, UINT base, LLCard* llcard) {
	mDspNumber = card_num;
	mCardIsReady = FALSE;
	mBaseAddress = base;
	int i;

	mAttachedCard = llcard;    //new LLCard(this);  // we should check that
//	mStrobeDelay = 0;						// the card is attached first...
	tccCard=NULL;
	for (i=0; i<4; i++) {
		DACscale[i] = 4096.0/2.5;
		DACcount[i] = 0;
	}
	for (i=0; i<6; i++) mFlags[i] = 0;

//	HAL * hal = theApp.GetHal();
//	hal->dspArray.Add(this);

	TRACE ("DSP created.\n");
}


DSP::~DSP() {
	UINT status;
	status= UnMapVXIAddress(windowid);
	delete mAttachedCard;
	delete tccCard;

//	HAL * hal = theApp.GetHal();
//	for (int i=0; i<hal->dspArray.GetSize(); i++) {
//		if (hal->dspArray[i] == this) hal->dspArray.RemoveAt(i);
//	}

	TRACE ("DSP destroyed.\n");
}

void DSP::operator=(const DSP & objectSrc) {
	mDspNumber = objectSrc.mDspNumber;
	mBaseAddress = objectSrc.mBaseAddress;
//	mLinksUsed = objectSrc.mLinksUsed;
	mRunMode = objectSrc.mRunMode;
	mTheName = objectSrc.mTheName;
	for (int i=0; i<6; i++)
		mFlags[i] = objectSrc.mFlags[i];
	for (int j=0; j<4; j++){
		DACcount[j] = objectSrc.DACcount[j];
		DACscale[j] = objectSrc.DACscale[j];
	}
	// FIXME copy LL card
}

void DSP::InitializeCard(const char * path) {
	int status = 0;
	double volts = 0;
	BootClass bootit;
   uint16 accessparms=2; // 2: A24, 3: A32
   int16 ret=1;
   uint32 timeout=1000L; // in milliseconds
	char buf[500];
	int i=0;

	mCardIsReady = FALSE;
	io_base = (dpr *) MapVXIAddress(accessparms, mBaseAddress,
		timeout, &windowid, &ret);
//	TRACE("WIN_ACCESS attribute = %i\n", VI_ATTR_WIN_ACCESS);
//	TRACE("VI_DEREF_ADDR = %i\n", VI_DEREF_ADDR);
   if (ret < 0) {
		i = sprintf(buf, "MapVXIAddress failed for address 0x%X\n", mBaseAddress);
		i += sprintf(buf+i, "with error code %i.\n", ret);
		switch (ret) {
		case -2:
			i += sprintf(buf+i, "Invalid address.  Make sure that the address in\n");
			i += sprintf(buf+i, "the Non-VXI device configuration editor matches\n");
			i += sprintf(buf+i, "the ALDAS configuration file.  Also check that the\n");
			i += sprintf(buf+i, "DIP switches are set to the right address\n");
			break;
		case -8:
			i += sprintf(buf+i, "Address has already been mapped.  Solution: \n");
			i += sprintf(buf+i, "quit the program and restart.\n");
			break;
		}
		AfxMessageBox(buf, MB_ICONSTOP|MB_OK);
		return;
   }

	// Before we download the system code, clear the flags.
	for (i=0; i<6; i++)
		SetFlag(i, 0);

   status = bootit.init_dsp(io_base, (char *) path);
	if (status > 0) {
		return;
	} else {
		mCardIsReady = TRUE;
	}

// If anything goes wrong during the initialization of the card, then
// an exception will be thrown.  We handle it here.


	try {
		// FIXME: test for LL card presence
		int status, cardAttached;
		ShortMessage(GET_DSP_STATUS, 0, 0, 0, 0);
		status = GetReply();
		cardAttached = TRUE; // this doen't seem to work: !(message_base[19] & 2);

		// If the LLCard object has been created, and the physical card is
		// attached, then Initialize it.
		if (mAttachedCard && cardAttached) {
			mAttachedCard->InitializeLLCard();
		}

		// Now initialize all the DSP parameters.  The flags and DAC values.
		for (i=0; i<6; i++)
			SetFlag( i, mFlags[i] );
		for (i=0; i<4; i++) {
			volts = (DACcount[i]-.5)/DACscale[i];
			SetDACVoltage( i, volts );
		}
	}
	catch (int initErrNumber) {
		i = sprintf(buf, "I'm sorry, I could not finish initializing the DSP\n");
		i += sprintf(buf+i, "card %s.  Possible solutions:\n", mTheName);
		i += sprintf(buf+i, "Please check that there is a clock plugged\n");
		i += sprintf(buf+i, "into the SLL card or that the TCC card is present.\n");
		i += sprintf(buf+i, "Click Retry to reboot the DSP, Cancel to give up.\n");
		if (AfxMessageBox(buf, MB_ICONSTOP|MB_RETRYCANCEL) == IDCANCEL) {
			mCardIsReady = FALSE;
		} else {
			UINT status;
			status= UnMapVXIAddress(windowid);
			InitializeCard(path);
		}
	}
}

CString & DSP::GetName() {
	return mTheName;
}

void DSP::SetName(CString &newname) {
	mTheName = newname;
}


void DSP::Serialize( CArchive& archive )
{
	char line[80];
	int i, j;
	float version=1.1;

	if( archive.IsStoring() ) {
      sprintf (line, "# ALDAS %5.1f\n", VERSION);
      archive.WriteString(line);
		sprintf (line, "%i mDspNumber\n", mDspNumber);
		archive.WriteString (line);
		int dipSwitch = mBaseAddress/0x80000;
		sprintf (line, "%X dip switch setting\n", dipSwitch);
		archive.WriteString (line);
		sprintf (line, "%i mRunMode\n", mRunMode);
		archive.WriteString (line);
		sprintf (line, "$%s\n", mTheName);
		archive.WriteString (line);
		for (i=0; i<6; i++){
			sprintf (line, "%i Flag %i\n", mFlags[i], i);
			archive.WriteString (line);
		}
		for (j=0; j<4; j++) {
			sprintf (line, "%i %f DAC %i\n", DACcount[j], DACscale[j], j);
			archive.WriteString (line);
		}
		if (mAttachedCard) {
			archive.WriteString ("1 LLCard follows:\n");
			mAttachedCard->Serialize(archive);
		} else {
			archive.WriteString ("0 LLCard.\n");
		}
		if (tccCard) {
			archive.WriteString ("1 TCCCard follows:\n");
			tccCard->Serialize(archive);
		} else {
			archive.WriteString ("0 TCCCard.\n");
		}
	}	else { // reading code:
		archive.ReadString (line, 80);
		if (line[0]=='#') { // version 
			sscanf (line, "# ALDAS %f", &version);
			archive.ReadString(line, 80);
		}
		sscanf (line, "%i", &mDspNumber);
		archive.ReadString (line, 80);
		int dipSwitch;
		sscanf (line, "%X", &dipSwitch);
		mBaseAddress = dipSwitch * 0x80000;
		archive.ReadString (line, 80);
		sscanf (line, "%i", &mRunMode);
		archive.ReadString (line, 80);
		mTheName=&(line[1]); // skip the $
		for (i=0; i<6; i++){
			archive.ReadString (line, 80);
			sscanf (line, "%i", &(mFlags[i]));
		}
		for (j=0; j<4; j++) {
			archive.ReadString (line, 80);
			sscanf (line, "%i %f", &(DACcount[j]), &(DACscale[j]));
		}
		
		archive.ReadString (line, 80);
		sscanf (line, "%i", &i);
		if (mAttachedCard) delete mAttachedCard;
		if (i>0) { // there is a LL card...
			mAttachedCard=new LLCard (this);
			mAttachedCard->Serialize(archive);
		} else { // no LLCard...
			mAttachedCard=NULL;
		}
		if (version>=1.2) {
			archive.ReadString (line, 80);
			sscanf (line, "%i", &i);
			if (tccCard) delete tccCard;
			if (i>0) { // there is a tcc card...
				tccCard=new TccCard (this);
				tccCard->Serialize(archive);
			} else { // no tccCard...
				tccCard=NULL;
//				tccCard=new TccCard(this);
			}
		}
	}
}

BOOL DSP::IsCardReady() {
	return mCardIsReady;
}

UINT DSP::GetBaseAddress() {
	return mBaseAddress;
}

void DSP::SetBaseAddress(UINT base) {
	mBaseAddress = base;
	mCardIsReady = FALSE;	// We must reinitialize the card now.
}

int DSP::GetDSPNumber() {
	return mDspNumber;
}

// LLCard information.
LLCard * DSP::GetLLCard() {
	return mAttachedCard;
}

void DSP::AttachLLCard(LLCard * llCard) {
	mAttachedCard = llCard;
}

void DSP::RemoveLLCard() {
	mAttachedCard = NULL;
}

// TCC information.
TccCard* DSP::GetTccCard() {
	return tccCard;
}

void DSP::AttachTccCard(TccCard * tcc) {
	tccCard = tcc;
}

void DSP::RemoveTccCard() {
	tccCard = NULL;
}

// Flag stuff.
int DSP::GetFlag( int flag ) {
  int val;
  
  if (flag > (NUM_FLAGS - 1))
    return ENOSIG;
  
  val = FLAG( flag);
  
  return val;
}

int DSP::SetFlag( int flag, int val )
{
	if (flag > (NUM_FLAGS - 1))
		return ENOSIG;

// FLAG actually writes the value into the DSPs DPRAM
	FLAG( flag) = val;
	mFlags[flag] = val;

	return SUCCESS;
}

int DSP::GetRunMode() {
	return mRunMode;
}

// DAC stuff.
double DSP::GetDACVoltage(int i)
{
	if ( (i>=0) && (i<=3) ) {
		return DACcount[i] / DACscale[i];
	} else {
		return 0.0;
	}
}

/********************** Sets the 4 possible DACs *****************/
/* DAC 0: V_att
/* DAC 1: V_ref
/* DAC 2: Not used by the hybrid - yet.
/* DAC 3: Calibration voltage - all 4 links.
/*****************************************************************/

void DSP::SetDACVoltage(int dac_channel, double voltage) {
	int dac_set_int;

	dac_set_int = (int) (DACscale[dac_channel] * voltage + 0.5);
	DACcount[dac_channel] = dac_set_int;
	
// Does channel one have the voltage divider??? CH
//	if(dac_channel == 1)
//		dac_set_int = dac_set_int/2;

	// testing the error message menu item.
	//dac_set_int = -500;

	ShortMessage(SET_DAC, 0, dac_channel, dac_set_int, 0);
	if (GetFlag(VERBOSE_FLAG) == 1)
		GetReply();
}


int DSP::GetDSPStatus() {
	int status;
	ShortMessage(GET_DSP_STATUS, 0, 0, 0, 0);
	status = GetReply();
	for (int i=0; i<4; i++) {
		DACcount[i] = message_base[11 + i];
	}
	return status;
}

UINT * DSP::DataTransfer(int link, int format){
// sends a message to transfer data from the dsp to the host.
// returns a pointer to the message area, where the data is stored.
// call CTakeDataDlg::AddHits(unsigned int * data) to read the hits.

	ShortMessage(TRANSFER_DATA, link, format, 0, 0);
	GetReply();
	return message_base;
}

void DSP::DoCalibrate(UINT link, UINT cb_number, int repetitions) {
	ShortMessage(CALIBRATE, link, repetitions, 1, cb_number);
	if (GetFlag(VERBOSE_FLAG) == 1)
		GetReply();
}

int DSP::ShortMessage(UINT ciw,  UINT link, UINT a1, UINT a2, UINT a3)
{
// the DSP knows its number, CMessage doesn't.
	return CMessage::ShortMessage(ciw, UINT(mDspNumber), link, a1, a2, a3);
}

int DSP::LongMessage(UINT ciw, UINT link, UINT* ptr, UINT arraysize,
							UINT nParams, UINT a1, UINT a2, UINT a3)
{
// the DSP knows its number, CMessage doesn't.
	return CMessage::LongMessage(ciw, UINT(mDspNumber), link, ptr, arraysize,
										 nParams,  a1, a2, a3);
	if (GetFlag(VERBOSE_FLAG) == 1)
		GetReply();
}
