/* Routines for fitting to gaussian or integral of gaussian */

/* from Numerical Recipies p. 545 */

#include "stdafx.h" // for afxmessagebox.
#include <math.h>
#include "fitLSQ.h"

#define  SQRT2PI   2.506628275
#define SQR(a) (sqrarg=(a),sqrarg*sqrarg)
#define  SQRT2  1.414213562


void fintgauss (float x, float a[], float * y, float dyda[], int na)
//	float  x;
//	float  a[]; 
//	float  *y;
//	float  dyda[];
//	int    na;     
// calculates  1-integrated gaussian
{
  float  x1 = ( x - a[1] ) / a[2];

  *y = 0.5 * erfcc (x1 / SQRT2);
  dyda[1] = exp (-0.5*x1*x1) / (SQRT2PI*a[2]);
  dyda[2] = dyda[1] * x1;
}

void intgausscomp (float x, float a[], float * y, float dyda[], int na)
{
  float  x1 = ( x - a[1] ) / a[2];

  *y = 1.0 - 0.5 * erfcc (x1 / SQRT2);
  dyda[1] = - exp (-0.5*x1*x1) / (SQRT2PI*a[2]);
  dyda[2] = - dyda[1] * x1;
}

void plateau (float x, float a[], float * y, float dyda[], int na) {
// to fit strobe delay scans...
//	float  x;
//	float  a[]; 
//	float  *y;
//	float  dyda[];

  float  x1 = ( x - a[1] ) / a[3];
  float  x2 = ( x - a[2] ) / a[3];

  *y = -0.5 * (erfcc (x1 / SQRT2) - erfcc (x2 / SQRT2));
  dyda[1] = -exp (-0.5*x1*x1) / ( SQRT2PI * a[3] );
  dyda[2] =  exp (-0.5*x2*x2) / ( SQRT2PI * a[3] );
  dyda[3] = dyda[1]*x1 + dyda[2]*x2;
}


void fguass (float x, float a[], float * y, float dyda[], int na)
//	float  x;
//	float  a[];
//	float  *y;
//	float  dyda[];
//	int    na;
	
	/* y(x;a) is a gaussian where the first value of a is the amplitude,
   		the second is the mean, and the third is the sigma.  
   		y = a[1]*exp((x-a[2])^2/2/a[3]^2)  */
{

  	float fac,ex,arg;
 
  	arg = (x - a[2]) / a[3];
  	ex  = exp ( -arg * arg / 2.0 ) ;
  	fac = a[1] * ex / SQRT2PI;
  	*y  = fac;

  	dyda[1] = ex / SQRT2PI;
  	dyda[2] = fac * arg / a[3];
  	dyda[3] = fac * arg * arg / a[3];

}

void parabola (float x, float a[], float * y, float dyda[], int na)
{
  float  x1 = ( x - a[1] ) / a[2];

  *y = a[1]+a[2]*x+a[3]*x*x;
  dyda[1] = 1.0;
  dyda[2] = x;
  dyda[3] = x*x;
}


void FitLine(float * x, float * y, float ndata,  float weight[], int mwt, 					 
				 float *a, float *b, float *siga, float *sigb, float *chi2, 
				 float *q) {

	int i;
	float wt, t, sxoss, sx=0.0, sy=0.0, st2=0.0, ss, sigdat;
	float sqrarg;
//	float gammq();

	*b = 0.0;
	if (mwt) {
		ss=0.0;
		for (i=0; i<ndata; i++) {
			wt=SQR(weight[i]);
			ss += wt;
			sx += x[i]*wt;
			sy += y[i]*wt;
		}
	} else {
		for (i=0; i<ndata; i++) {
			sx += x[i];
				sy += y[i];
		}
		ss=ndata;
	}
	sxoss = sx/ss;
	if (mwt) {
		for (i=0; i<ndata; i++) {
			t=(x[i]-sxoss)*weight[i];
			st2 += t*t;
			*b += t*y[i]* weight[i];
		}
	} else {
		for (i=0; i<ndata; i++) {
			t=x[i]-sxoss;
			st2 += t*t;
			*b += t*y[i];
		}
	}
	*b /= st2;
	*a = (sy-sx*(*b))/ss;
	*siga=sqrt((1.0+sx*sx/(ss*st2))/ss);
	*sigb=sqrt(1.0/st2);
	*chi2=0.0;
	if (mwt==0) {
		for (i=0; i<ndata; i++)
			*chi2 += SQR (y[i]-(*a)-(*b)*x[i]);
		*q=1.0;
		sigdat=sqrt((*chi2)/(ndata-2));
		*siga *=sigdat;
		*sigb *=sigdat;
	} else {
		for (i=0; i<ndata; i++)
			*chi2 += SQR((y[i]-(*a)-(*b)*x[i])*weight[i]);
//		*q=GammQ(0.5*(ndata-2),0.2*(*chi2));
		*q = 0;
	}
}

