#include "stdafx.h"
#include <math.h>

//#include "stdio.h"
#include <stdlib.h>
#include <malloc.h>

/* The routines listed below are used by many of the Recipes.  Most of the routines
are used to allocate and deallocate memory for vectors and matricies, as explained in
detail in sec. 1.2, p. 15-20.  All memory allocation for Numerical Recipes functions
are handled using these functions.  */

/* Taken from appendix D, pp. 705-709. */


/* =================================================== */
/*           nerror                                    */
/* =================================================== */


void nrerror (char  *error_text)
{
	TRACE("Utility error: %s\n", error_text);
}


/* =================================================== */
/*                                                     */
/* =================================================== */


float *vector (int nl, int nh)
{

	float  *v;
	v = (float*)malloc((size_t) (nh-nl+1)*sizeof(float));
	if(!v) nrerror("allocation faiure in vector()");
	return v-nl;
}


void free_vector(float *v, int nl, int nh)
{
	free((char *) (v+nl));
}



int *ivector (int nl, int nh)
{
	int *v;
	v=(int *)malloc((size_t) (nh-nl+1)*sizeof(int));
	if(!v) nrerror("allocation failure in ivector()");
	return v-nl;
}


void free_ivector(int *v, int nl, int nh)
{
	free ((char *) (v+nl));
}


/* =================================================== */
/*                                                     */
/* =================================================== */


float **matrix(int nrl, int nrh, int ncl, int nch)
{
	
	int i;
	float **m;
	m = (float **)malloc((size_t) (nrh-nrl+1)*sizeof(float*));
	if (!m) nrerror("allocation failure 1 in matrix()");
	m -= nrl;
	for(i=nrl;i<=nrh;i++) {
		m[i]=(float *) malloc((size_t) (nch-ncl+1)*sizeof(float));
		if (!m[i]) nrerror("allocation failure 2 in matrix()");
		m[i] -= ncl;
	}
	return m;
}



void free_matrix (float **m, int nrl, int nrh, int ncl, int nch)
{
	
	int i;
	
	for(i=nrh;i>=nrl;i--) free((char *) (m[i]+ncl));
	free((char*) (m+nrl));
}