#include "stdafx.h"
#include <math.h>
#include "fitLSQ.h"

float gammln(float xx)
//float xx;
/* returns natural log of gamma function.  for xx > 0.  Full accuracy is obtained  for
   xx>1.  */
{
   float x,tmp,ser;
   static float cof[6]={76.18009173,-86.50532033,24.01409822,
       -1.231739516,0.120858003e-2, -0.536382e-5};
   int j;
   x=xx-1.0;
   tmp=x+5.5;
   tmp -= (x+0.5)*log(tmp);
   ser=1.0;
   for (j=0;j<=5;j++) {
       x += 1.0;
       ser += cof[j]/x;
   }
   return -tmp+log(2.50662827465*ser);
}

float factrl(int n)
//int n;
/* Returns the value n! as a floating point number.  */
{
	static int ntop=4;
	static float a[33] = {1.0,1.0,2.0,6.0,24.0};  /* table filled as required */
	int j;
//	float gammln();
//	void nrerror();
	
	if (n < 0) nrerror("Negative factorial in routine FACTRL");
	if (n > 32) return exp(gammln(n+1.0));
	/* Larger value than size of table is required.  Actually, this big a value will
	overflow many computers, but no harm in trying.  */
	while (ntop<n) {
		j=ntop++;
		a[ntop]=a[j]*ntop;
	}
	return a[n];
}

float bico(int n, int k)
//int n,k;
/* Returns the binomial coefficient (n k) as a floating-point number.  */
{
	float factln(int);
	
	return floor(0.5+exp(factln(n)-factln(k)-factln(n-k)));
	/* floor cleans up roundoff errors */
}

float factln(int n)
//int n;
/* return ln(n!) */
{
	static float a[101];
//	float gammln();
//	void nrerror();
	
	if(n < 0) nrerror("Negative factorial in routine FACTLN");
	if (n <= 1) return 0.0;
	if (n <= 100) return a[n] ? a[n] : (a[n]=gammln(n+1.0));
	else return gammln(n+1.0);
}

float beta(float z, float w)
//float z,w;
/* returns the value of the beta function B(z,w). */
{
	float gammln(float);

	return exp(gammln(z)+gammln(w)-gammln(z+w));
}

float gammp(float a, float x)
//float a,x;
/* returns the incomplete gamma function P(a,x) */
{
   float gamser,gammcf,gln;
 //  void gser(),gcf(),nrerror();
   
   if (x < 0.0 || a <= 0.0) nrerror("Invalid arguments in routine GAMMP");
   if(x < (a+1.0)) {
       gser(&gamser,a,x,&gln);
       return gamser;
   } else {
       gcf(&gammcf,a,x,&gln);
       return 1.0-gammcf;
   }
}

float gammq(float a, float x)
//float a,x;
/* Returns the incomplete gamma function Q(a,x) = 1-P(a,x). */
{
   float gamser, gammcf, gln;
  // void gcf(),gser(), nrerror();
   if (x < 0.0 || a <= 0.0) nrerror("Improper arguments to GAMMQ\n");
   if (x < (a+1.0)) {
       gser(&gamser,a,x,&gln);
       return 1.0-gamser;
   } else {
       gcf(&gammcf,a,x,&gln);
       return gammcf;
   }
}

#define ITMAX 100
#define EPS 3.0e-7

void gser(float * gamser, float a, float x, float * gln)
//float a,x,*gamser,*gln;
/* Returns the incomplete gamma function P(a,x) evaluated by its series 
   representation as gamser.  */
{
   int n;
   float sum,del,ap;
 //  float gammln();
 //	void nerror ()
   *gln=gammln(a);
   if (x<= 0.0) {
       *gamser=0.0;
       return;
   } else {
       ap=a;
       del=sum=1.0/a;
       for (n=1;n<=ITMAX;n++) {
           ap += 1.0;
           del *= x/ap;
           sum += del;
           if (fabs(del) < fabs(sum)*EPS) {
               *gamser=sum*exp(-x+a*log(x)-(*gln));
               return;
           }
       }
       nrerror("a too large, ITMAX too small in routine GSER");
       return;
   }
}

void gcf(float * gammcf, float a, float x, float * gln)
//float a,x,*gammcf,*gln;
{
   int n;
   float gold=0.0,g,fac=1.0,b1=1.0;
   float b0=0.0,anf,ana,an,a1,a0=1.0;
   float gammln(float);

   *gln=gammln(a);
   a1=x;
   for (n=1;n<=ITMAX;n++) {
       an=(float) n;
       ana=an-a;
       a0=(a1+a0*ana)*fac;
       b0=(b1+b0*ana)*fac;
       anf=an*fac;
       a1=x*a0+anf*a1;
       b1=x*b0+anf*b1;
       if (a1) {
           fac=1.0/a1;
           g=b1*fac;
           if (fabs((g-gold)/g) < EPS) {
               *gammcf=exp(-x+a*log(x)-(*gln))*g;
               return;
           }
        gold=g;
        }
   }
   nrerror("a too large, ITMAX too small in routine GCF");
   return;
}
