/* Routines for least-squares fits */

/* requires gamma.c and gaussj.c */

/* from Numerical Recipies Chapter 14. pp. 525-545 */
#include "stdafx.h"

#include <math.h>
//#include <stdio.h>
//#include <stdlib.h>

#include "fitLSQ.h"


static float sqrarg;
#define SQR(a) (sqrarg=(a), sqrarg*sqrarg)

void fit(float x[], float y[], int ndata, float sig[], int mwt, float * a, float * b,
			float * siga, float * sigb, float * chi2, float * q)
//float x[],y[],sig[],*a,*b,*siga,*sigb,*chi2,*q;
//int ndata,mwt;
{
}

void lfit(float x[], float y[], float sig[], int ndata, float a[], int ma,
			 int lista[], int mfit,float ** covar,float * chisq,
			 void (*funcs)(float,float *,int))
//int ndata,ma,lista[],mfit;
//float x[],y[],sig[],a[],**covar,*chisq;
//void (*funcs)(float,float *,int);  /* non-ANSI:  void (*funcs)(); */
{
}


void covsrt(float ** covar, int ma, int lista[], int mfit)
//float **covar;
//int ma,lista[],mfit;
/* given covariance matrix covar[1..ma][1..ma] of a fit for mfit of ma total 
   parameters, and their orderling lista[1..ma], repack the covar matrix to
   the true order of the parameters.
   Elements associated with fixed parameters will be zero.  */
{
	int i,j;
	float swap;
	
	for (j=1;j<ma;j++)
		for(i=j+1;i<=ma;i++) covar[i][j] = 0.0;
	for (i=1;i<mfit;i++)
		for (j=i+1;j<=mfit;j++) {
			if (lista[j] > lista[i])
				covar[lista[j]][lista[i]]=covar[i][j];
			else
				covar[lista[i]][lista[j]]=covar[i][j];
		}
	swap=covar[1][1];
	for (j=1;j<=ma;j++) {
		covar[1][j]=covar[j][j];
		covar[j][j]=0.0;
	}
	covar[lista[1]][lista[1]]=swap;
	for (j=2;j<=mfit;j++) covar[lista[j]][lista[j]]=covar[1][j];
	for (j=2;j<=ma;j++)
		for (i=1;i<=j-1;i++) covar[i][j]=covar[j][i];
}

#define TOL 1.0e-5

void svdfit(float x[], float y[], float sig[], int ndata, float a[], int ma,
				float ** u, float ** v, float w[], float * chisq,
				void (*funcs)(float,float *,int))
//float x[],y[],sig[],a[],**u,**v,w[],*chisq;
//int ndata,ma;
//void (*funcs)(float,float *,int);  /* non-ANSI:  void (*funcs)(); */
{
}

void svdvar(float ** v, int ma, float w[], float ** cvm)
//float **v,w[],**cvm;
//int ma;
/* to evaluate the covariance matrix cvm[1..ma][1..ma] of the fit for ma parameters
obtained by svdfit, call this routine with matricies v[1..ma][1..ma], w[1..ma] as 
returned from svdfit. */
{
	int k,j,i;
	float sum,*wti;
	// float *vector();
	// void free_vector();
	
	wti=vector(1,ma);
	for (i=1;i<=ma;i++) {
		wti[i]=0.0;
		if (w[i]) wti[i]=1.0/(w[i]*w[i]);
	}
	for (i=1;i<=ma;i++) {
		for (j=1;j<=i;j++) {
			for (sum=0.0,k=1;k<=ma;k++) sum += v[i][k]*v[j][k]*wti[k];
			cvm[j][i]=cvm[i][j]=sum;
		}
	}
	free_vector(wti,1,ma);
}

void fpoly(float x, float p[], int np)
//float x,p[];
//int np;
/* Fitting routine for a polynomial of degree NP-1, with coefficients in the array
p[1..np]. */
{
	int j;
	
	p[1]=1.0;
	for (j=2;j<=np;j++) p[j]=p[j-1]*x;
}

void fleg(float x, float pl[], int nl)
//float x,pl[];
//int nl;
/* Fitting routine for an expansion with nl Legendre polynomials pl, evaluated
using the recurrence relation as in section 4.5 */
{
	int j;
	float twox,f2,f1,d;
	
	pl[1]=1.0;
	pl[2]=x;
	if (nl > 2) {
		twox =2.0*x;
		f2=x;
		d=1.0;
		for (j=3;j<=nl;j++) {
			f1=d;
			d += 1.0;
			f2 += twox;
			pl[j]=(f2*pl[j-1]-f1*pl[j-2])/d;
		}
	}
}


void mrqcof(float x[], float y[], float sig[], int ndata, float a[], int ma,
				int lista[], int mfit, float ** alpha, float beta[], float * chisq, 
				void (*funcs)(float,float *,float *,float *,int))
//float x[],y[],sig[],a[],**alpha,beta[],*chisq;
//int ndata,ma,lista[],mfit;
//void (*funcs)(float,float *,float *,float *,int);
/* used by mrqmin to evaluate the linearized fitting matrix alpha[1..mfit][1..mfit] and
   vector beta[1..mfit] as in (14.4.8). */
{
   	int k,j,i;
   	float ymod,wt,sig2i,dy,*dyda;

   	dyda=vector(1,ma);
   	for(j=1;j<=mfit;j++) {
   		for(k=1;k<=j;k++) alpha[j][k] = 0.0;
   		beta[j]=0.0;
		}
   	*chisq=0.0;
   	for(i=1;i<=ndata;i++) {
   		(*funcs)(x[i],a,&ymod,dyda,ma);
   		sig2i=1.0/(sig[i]*sig[i]);
   		dy=y[i]-ymod;
   		for (j=1;j<=mfit;j++) {
   			wt = dyda[lista[j]]*sig2i;
   			for(k=1;k<=j;k++)
   				alpha[j][k] += wt*dyda[lista[k]];
   			beta[j] += dy*wt;
   		}
   		(*chisq) += dy*dy*sig2i;
   	}
   	for(j=2;j<=mfit;j++)
   		for(k=1;k<=j-1;k++) alpha[k][j] = alpha[j][k];
   	free_vector(dyda,1,ma);
}

/* ====================================================== */
/*     mrqmin                                             */
/* ====================================================== */													  

int mrqmin (float  *x,
			float  *y,
			float  *sig,
			int     ndata,
			float  *a,
			int     ma,
			int     *lista,
			int     mfit,
			float  **covar,
			float  **alpha,
			float  *chisq,
			void    (*funcs)(float,float *,float *,float *,int),
			float  *alamda )

/*
int mrqmin(x,y,sig,ndata,a,ma,lista,mfit,covar,alpha,chisq,funcs,alamda)
float x[],y[],sig[],a[],**covar,**alpha,*chisq,*alamda;
int ndata,ma,lista[],mfit;
void (*funcs)(float,float *,float *,float *,int); 
*/

/* non-ANSI:  void (*funcs)(); */

													  
{
	
	int k, kk, j, ihit;
	static float *da, *atry,**oneda,*beta,ochisq;
	//float *vector(),**matrix();
	//void mrqcof(),covsrt(),nrerror(),free_matrix(),free_vector();
	//int gaussj();
	
	if(*alamda < 0.0) {
		oneda = matrix(1,mfit,1,1);
		atry=vector(1,ma);
		da=vector(1,ma);
		beta=vector(1,ma);
		kk=mfit+1;
		for(j=1;j<=ma;j++) {
			ihit=0;
			for(k=1;k<=mfit;k++)
				if(lista[k]==j) ihit++;
			if(ihit==0)
				lista[kk++] = j;
			else if (ihit>1) {
				nrerror("Bad LISTA permutation in MRQMIN-1");
				free_vector(beta,1,ma);
				free_vector(da,1,ma);
				free_vector(atry,1,ma);
				free_matrix(oneda,1,mfit,1,1);
				return -1;
			}
		}
		if(kk!=ma+1) {
			nrerror("Bad LISTA permutation in MRQMIN-2");
			free_vector(beta,1,ma);
			free_vector(da,1,ma);
			free_vector(atry,1,ma);
			free_matrix(oneda,1,mfit,1,1);
			return -1;
		}
		*alamda = 0.001;
		mrqcof(x,y,sig,ndata,a,ma,lista,mfit,alpha,beta,chisq,funcs);
		ochisq=(*chisq);
	}
	for(j=1;j<=mfit;j++) {
		for(k=1;k<=mfit;k++) covar[j][k] = alpha[j][k];
		covar[j][j]=alpha[j][j]*(1.0+(*alamda));
		oneda[j][1] = beta[j];
	}
	if(gaussj(covar,mfit,oneda,1)==-1) {
		free_vector(beta,1,ma);
		free_vector(da,1,ma);
		free_vector(atry,1,ma);
		free_matrix(oneda,1,mfit,1,1);
		return -1;
	}
	for(j=1;j<=mfit;j++)
		da[j]=oneda[j][1];
	if (*alamda == 0.0) {
		covsrt(covar,ma,lista,mfit);
		free_vector(beta,1,ma);
		free_vector(da,1,ma);
		free_vector(atry,1,ma);
		free_matrix(oneda,1,mfit,1,1);
		return 0;
	}
	for(j=1;j<=ma;j++) atry[j]=a[j];
	for(j=1;j<=mfit;j++) {
		atry[lista[j]] = a[lista[j]]+da[j];
	}
	mrqcof(x,y,sig,ndata,atry,ma,lista,mfit,covar,da,chisq,funcs);
	if(*chisq < ochisq) {
		*alamda *= 0.1;
		ochisq = (*chisq);
		for (j=1;j<=mfit;j++) {
			for(k=1;k<=mfit;k++) alpha[j][k] = covar[j][k];
			beta[j] = da[j];
			a[lista[j]]=atry[lista[j]];
		}
	} else {
		*alamda *= 10.0;
		*chisq = ochisq;
	}
	return 0;
}

