/* This is "strgeom.h" */

/* Here we define the structure used to configure
 * the DSP readout.  This info will also be needed
 * to unpack the raw data.  The format is to have
 * a structure for each dsp that contains the
 * following information for each link (each
 * dsp can read up to four links with 1 link
 * corresponding to one hack):
 *   plane_num[MAXLINK] : should correspond to same
 *         numbering convention as in Planeinfo
 *   unpk_algorp[MAXLINK]: tells the unpacking
 *         algorithm how the strips are connected
 *   hi[MAXLINK]: says whether bit 1 on or off
 *         corresponds to a hit
 *   nchips: number of chips on the link (must
 *    be the same for all links on a dsp)
 *
 * The structure bina_config contains the
 * run number and the number of time buckets
 * being read out as well as the Dsp_config
 * for each Dsp.
 */
 
#define MAXINFO 12
#define MAXDSP 6
#define MAXLINK 4
static int ll_counter[6] = {20, 37, 52, 68, 88, 104};

typedef struct {
  int link[MAXLINK]; 
  int unpk_algor[MAXLINK];
  int hi[MAXLINK]; 
  int nchips; 
  float sysdac[4];
  int lldac[4];
  char sys_name[40];	     /* name of system file to download */
  char trig_control[40];  /* name of trigger control file */
  int base_address;
} Dsp_config;

typedef struct { 
  int ndsp;
  Dsp_config dsp_config[MAXDSP];
} Binary_config;

// This is the class ConfigurationInfo

class ConfigInfo {
public:
	ConfigInfo();
	~ConfigInfo();
	void OpenConfigFile(void);
	void CloseConfigFile(void);
	int GetNumDSP(void);
	char * GetPathToSystem(void);
	unsigned int GetBaseAddress( int );

private:
	void ReadPlaneInfo( void );
	FILE * OpenFileForReading( char * );
	int InfoNumLinks( int );
	int InfoLinkActive( int, int );

	Binary_config current_config;
	int num_dsp;
	int NumberOfPlanes;
	FILE * config_file_ptr;
};