// Class LLCard:
#include "stdafx.h"

#include "module.h"
#include "lbic.h"

#include "llcard.h"
#include "message.h"
#include "dspcard.h"
#include "aldas.h"

extern CALDASApp theApp;

LLCard::LLCard(DSP * myParent) {
	mStrobeDelay = 0;	
	mCounter=0;
	for (int i=0; i<4; i++) {
		LLDACscale[i] = 4096.0/2.5;
		LLDACcount[i] = 0;

		module[i] = NULL;
	}
	ParentDSP = myParent;

	TRACE("LLCard created.\n");
}


LLCard::~LLCard() {
	for (int i=0; i<4; i++)
		delete module[i];

	TRACE("LLCard destroyed.\n");
}

void LLCard::AttachModule(Module * m, int link) {
	module[link] = m;
}

void LLCard::RemoveModule(int link) {
	module[link] = NULL;
}

Module * LLCard::GetModule(int link) {
	return module[link];
}

void LLCard::Serialize( CArchive& archive )
{
	char line[80];
	int i, j;
	float version=1.1;

 	if( archive.IsStoring() ) {
      sprintf (line, "# ALDAS %5.1f\n", VERSION);
      archive.WriteString(line);
		sprintf (line, "%i Strobe Delay\n", mStrobeDelay);
		archive.WriteString (line);
		sprintf (line, "%i Counter\n", mCounter);
		archive.WriteString (line);

		for (i=0; i<4; i++) {
			sprintf (line, "%i %f LL dac %i\n", LLDACcount[i], LLDACscale[i], i);
			archive.WriteString (line);
			sprintf (line, "$%s\n", cbFileName[i]);
			archive.WriteString(line);
		}
		for (i=0; i<4; i++) {
			if (module[i]) {
				archive.WriteString("1 Module:\n");
				module[i]->Serialize(archive);
			} else {
				archive.WriteString("0 Module.\n");
			}
		}

	} else {
		archive.ReadString (line, 80);
		if (line[0]=='#') { // version 
			sscanf (line, "# ALDAS %f", &version);
			archive.ReadString(line, 80);
		}
		sscanf (line, "%i",  &mStrobeDelay);
		archive.ReadString (line, 80);
		sscanf (line, "%i",  &mCounter);
		for (i=0; i<4; i++) {
			archive.ReadString (line, 80);
			sscanf (line, "%i %f",  &(LLDACcount[i]), &(LLDACscale[i]));
			archive.ReadString(line, 80);
			cbFileName[i]=&(line[1]); // skip the $
		}
		for (i=0; i<4; i++) {
			archive.ReadString (line, 80);
			sscanf (line, "%i",  &j);
			if (j>0) { // there is a hybrid...	
				module[i]=new LBIC;
				module[i]->Serialize(archive);
				module[i]->llCard = this;
			} else { //there is no hybrid:
				module[i]=NULL;
			}
		}
	}
}

void LLCard::InitializeLLCard() {
	int i;
	for (i=0; i<4; i++) 
		SetLLDACVoltage(i, LLDACcount[i] / LLDACscale[i]);
	// fifos..
	ResetFifos();
	for (i=0; i<4; i++) {
		if (cbFileName[i] != "")
			LoadControlBlock(i, cbFileName[i]);
	}
	SetCounterValue(mCounter);
	SetStrobeDelay(mStrobeDelay);
}

void LLCard::InitializeModule() {
	CString cbOld; // name of old control block # 0
	int success;										// the HAC is initially in a bad state.
	cbOld = GetControlBlock(0);
	success = LoadControlBlock(0, theApp.cbHacReset);	// this resets it.
	if (success)
		ExecuteControlBlock(0, 50);
	LoadControlBlock(0, cbOld); // restore original cb
}

void LLCard::SetLLDACVoltage(int dac_channel, double voltage) {
	int dac_set_int;
	int result;
	// do some error checking...
	dac_set_int = (int) (LLDACscale[dac_channel] * voltage + 0.5);
	LLDACcount[dac_channel] = dac_set_int;
	
	result = ParentDSP->ShortMessage(SET_LL_DAC, 0, dac_channel, dac_set_int, 0);
	if (ParentDSP->GetFlag(VERBOSE_FLAG) == 1)
		ParentDSP->GetReply();

}


double LLCard::GetLLDACVoltage(int the_dac) {
	if ( (the_dac>=0) && (the_dac<=3) ) {
		return (float) LLDACcount[the_dac] / LLDACscale[the_dac];
	} else {
		return 0.0;
	}
}


int LLCard::LoadControlBlock(int fifoNumber, CString fileName) {

//	fileNamePath += fileName;


	CStdioFile  cb_file;

	if( !cb_file.Open ((char *) (LPCTSTR) fileName, CFile::modeRead) ) {
		TRACE ("Unable to open file %s \n", fileName);
		return 0;
	}

	// --------------------
	//   read file 
	// --------------------


	char  buffer[101];
	int   nRepeat, setCal, setCom;
	int   lineCount = 0;
	UINT   i, blkData[2048];


	for ( i = 0; i < 2048; i++ )
		blkData[i] = 0;


	while ( cb_file.ReadString (buffer, 100) )  {

		if ( buffer[0] != '#' )  {

			if ( sscanf (buffer, "%d %x %x", &nRepeat, &setCal, &setCom) == 3 )  {

				setCal  = 0x000000f0 & (setCal << 4 );
				setCal |= 0x0000000f & setCom;

				while ( lineCount < 2048 && nRepeat > 0 )  {
					blkData [lineCount] = setCal;
					++lineCount;	
					--nRepeat;
				}

			}
		}
	}

	cb_file.Close ();

	int result;
	result = ParentDSP->LongMessage(LOAD_CONTROL_BLOCK, 0, 
		 blkData, sizeof(blkData)/sizeof(int), 1, fifoNumber, 0, 0);
	if (ParentDSP->GetFlag(VERBOSE_FLAG) == 1)
		ParentDSP->GetReply();

	if (result == SUCCESS)
		cbFileName[fifoNumber] = fileName;
	return 1;

}

CString LLCard::GetControlBlock(int i) {
	if ((i>=0)&&(i<=3)) {
		return cbFileName[i];
	} else {
		return "External trigger";
	}
}

int LLCard::ResetFifos() {
	ParentDSP->ShortMessage(RESET_FIFOS, 0, 0, 0, 0);
	if (ParentDSP->GetFlag(VERBOSE_FLAG) == 1)
		ParentDSP->GetReply();
	return 1;
}

int LLCard::ExecuteControlBlock(int fifo_number, int num_reps) {
	ParentDSP->ShortMessage(EXECUTE_CONTROL_BLOCK, 0, fifo_number, num_reps, 0);
	if (ParentDSP->GetFlag(VERBOSE_FLAG) == 1)
		ParentDSP->GetReply();
	return 1;
}

// Strobe delay stuff.
int LLCard::GetStrobeDelay() {
	return mStrobeDelay;
}


void LLCard::SetStrobeDelay(int inDelay) {
	int status = 0;
	ParentDSP->ShortMessage(SET_STROBE_DELAY, 0, inDelay, 0, 0);

	if(ParentDSP->GetFlag(VERBOSE_FLAG) != 0) {
		status = ParentDSP->GetReply();
	}
	mStrobeDelay = inDelay;
}

void LLCard::SetCounterValue(int i){
	int status = 0;
	ParentDSP->ShortMessage(SET_COUNTER_VAL, 0, i, 0, 0);

	if(ParentDSP->GetFlag(VERBOSE_FLAG) != 0) {
		status = ParentDSP->GetReply();
	}
	mCounter = i;
}


/*void LLCard::DisEnableLink(int link, int i){
	// i>0: enable, i<0: disable link
	int status = 0;
	ParentDSP->ShortMessage(DIS_ENABLE_LINK, link, i, 0, 0);

	if(ParentDSP->GetFlag(VERBOSE_FLAG) != 0) {
		status = ParentDSP->GetReply();
	}
}*/


int LLCard::GetCounter(){
	return mCounter;
}

// This isn't behaving the way it should!  I should get
// an error message if the LL card isn't installed, but
// I don't!
int LLCard::GetLLStatus() {
	int status;

	ParentDSP->ShortMessage(GET_LL_STATUS, 0, 0, 0, 0);
	status = ParentDSP->GetReply();
	for (int i=0; i<4; i++) {
		LLDACcount[i] = ParentDSP->message_base[11 + i];
	}
	mStrobeDelay=ParentDSP->message_base[15];
	mCounter=ParentDSP->message_base[16];

	/*if(ParentDSP->GetFlag(VERBOSE_FLAG) != 0) {
		status = ParentDSP->GetReply();
	}*/
	return 0;
}